
 UNIT UnForCoo ; { simple procedure to change coordinate systems }

 INTERFACE

 Uses
   UnConTyp ; { for type TMatr33 TVect3  const JD2000 JulianC }

 Procedure ClcSpherCoor( Pos : TVect3 ;
                         Var Alfa , Delta, Ro : Extended );

 Procedure ClcSpherCoorInDegree( Pos : TVect3 ;
                                 Var Alfa , Delta, Ro : Extended );

 Procedure GeoSpherFromCart ( x , y , z          : Extended ;
                              Var   f , v ,  h   : Extended ) ;

 Procedure GeoCartFromSpher ( f , v , h         : Extended ;
                              Var   x , y , z   : Extended ) ;

 Procedure DescFromSpherCoor ( Az , Alt , Ro    : Extended ;
                               Var  PosDescart  : TVect3 ) ;

 Procedure GeodeticSpher ( x , y , z          : Extended ;   { meter }
                           Var   f , v ,  h   : Extended ) ; { radian km }

 IMPLEMENTATION

 Uses
   UnForFun ; { function DATan2 ATanDegree for example }

 { ClcSpherCoor is a procedure
   to obtain the spherical coordinates of the planet
      from cartesian ones }

 Procedure ClcSpherCoor( Pos : TVect3 ;
                         Var Alfa , Delta, Ro : Extended );
 Var
   xr : TVect3;
   r,rr,q,qq,p,f,s,c,a,b : Extended;
   i : Integer;
 Begin
   for i:=1 to 3 do  xr[i]:=Pos[i];
   qq:=Sqr(xr[1])+Sqr(xr[2]);
   rr:=qq+Sqr(xr[3]);
   r:=Sqrt(rr);
   q:=Sqrt(qq);
   p:=1/q;
   f:=1/r;
   c:=p*xr[1];
   s:=p*xr[2];
   a:=DATan2(s,c); { from unit UnForFun }
   c:=q*f;
   s:=xr[3]*f;
   b:=DATan2(s,c);
   if  a < 0  then  a:=a+Pi22;
   Alfa:=a;
   Delta:=b;
   Ro:=r;
 End;

 { ClcSpherCoorInDegree is a procedure
   to obtain the spherical coordinates of the planet from cartesian ones }

 Procedure ClcSpherCoorInDegree ( Pos : TVect3 ;
                                  Var Alfa , Delta, Ro : Extended );
 Var
   xr : TVect3;
   r,rr,q,qq,p,f,s,c : Extended;
   i : Integer;
 Begin
   for i:=1 to 3 do  xr[i]:=Pos[i];
   qq:=Sqr(xr[1])+Sqr(xr[2]);
   rr:=qq+Sqr(xr[3]);
   r:=Sqrt(rr);
   q:=Sqrt(qq);
   p:=1/q;
   f:=1/r;
   c:=p*xr[1];
   s:=p*xr[2];
   Alfa:=ATanDegree(s,c); { from unit UnForFun }
   c:=q*f;
   s:=xr[3]*f;
   Delta:=ATanDegree(s,c);
   Ro:=r;
 End;

 { to obtain spherical coordinates ( h in metr )
      of the station from the cartesian ones in metr }

 Procedure GeoSpherFromCart ( x , y , z          : Extended ;
                              Var   f , v ,  h   : Extended ) ;
 Var
   irf                : Byte ;
   a,e,r,
   pp,hg,gg,sv,cv,
   sf,cf,sff,cff,cc   : Extended ;
 Begin
   r:=6378144.11; { in metr }
   a:=1.0/298.257;
   e:=2*a-a*a;
   pp:=Sqrt(x*x+y*y);
   If pp < 1.0e4
     Then
       r:=1.0e-3*r; { in km }
   cv:=x/pp;
   sv:=y/pp;
   hg:=0.0;
   For irf:=1 To 4 Do
     Begin
       sff:=z*(1+hg);
       cff:=pp*(1-e+hg);
       cc:=Sqrt(sff*sff+cff*cff);
       sf:=sff/cc;
       cf:=cff/cc;
       gg:=r/Sqrt(1-e*sf*sf);
       hg:=pp/(gg*cf)-1.0;
     End;
   f:=ATanDegree(sf,cf); { from unit UnForFun }
   v:=ATanDegree(sv,cv);
   h:=hg*gg;
 End;

 { to obtain cartesian coordinates in metr
      of the station from the spherical ones }

 Procedure GeoCartFromSpher ( f , v , h         : Extended ;
                              Var   x , y , z   : Extended ) ;
 Var
   a,e,r,q,
   ff,vv,sf,cf,gg  : Extended ;
 Begin
   r:=6378144.11;
   a:=1.0/298.257;
   e:=2*a-a*a;
   q:=1-e;
   ff:=GraRad*f;
   vv:=GraRad*v;
   sf:=Sin(ff);
   cf:=Cos(ff);
   gg:=r/Sqrt(1-e*sf*sf);
   x:=(gg+h)*cf*Cos(vv);
   y:=(gg+h)*cf*Sin(vv);
   z:=(gg*q+h)*sf;
 End;

 Procedure DescFromSpherCoor ( Az , Alt , Ro    : Extended ;
                               Var  PosDescart  : TVect3 ) ;
 Var
   a,b,cb  : Extended ;
 Begin
   a:=GraRad*Az;
   b:=GraRad*Alt;
   cb:=Cos(b);
   PosDescart[1]:=Ro*Cos(a)*cb;
   PosDescart[2]:=Ro*Sin(a)*cb;
   PosDescart[3]:=Ro*Sin(b);
 End;

 { to obtain spherical coordinates
      latitude  f in radian  but -Pi/2 <= latitude <= +Pi/2
      longitude v in radian  but  0 <= longitude <= +2*Pi
      height    h in kilometer
      of the station from the cartesian ones in metr }

 Procedure GeodeticSpher ( x , y , z          : Extended ;   { meter }
                           Var   f , v ,  h   : Extended ) ; { radian km }
 Var
   irf                : Byte ;
   a,e,r,
   pp,hg,gg,sv,cv,
   sf,cf,sff,cff,cc   : Extended ;
 Begin
   r:=6378144.11; { in metr }
   a:=1.0/298.257;
   e:=2*a-a*a;
   pp:=Sqrt(x*x+y*y);
   cv:=x/pp;        { cosinus longitude }
   sv:=y/pp;        {   sinus longitude }
   hg:=0.0;
   For irf:=1 To 4 Do
     Begin          { some iterations }
       sff:=z*(1+hg);
       cff:=pp*(1-e+hg);
       cc:=Sqrt(sff*sff+cff*cff);
       sf:=sff/cc;  { cosinus latitude }
       cf:=cff/cc;  {   sinus latitude }
       gg:=r/Sqrt(1-e*sf*sf);
       hg:=pp/(gg*cf)-1.0;
     End;
   f:=DATan2(sf,cf); { latitude in radian from unit UnForFun }
   v:=DATan2(sv,cv); { longitude in radian function UnForFun }
   if  v < 0.0  then { may be negative value of longitude }
     v:=Pi22+v;      { 0 <= longitude <= 2*Pi radian unConTyp }
   h:=1.0e-3*hg*gg;  { height in kilometer }
 End;

 END.