(*  from unit unfortyq
type
  TRecObs = record             // simple record for observations
              ns  : LongInt ;  // number of the satellite
              no  : SmallInt ; // number of the observatory
              { moment in UTC  of firing on the observatory }
              dm  : extended ; // day as julian day
              sp  : Extended ; // part of day second of time
              ds  : Extended ; // time of delay two way in second
              lw  : Word ;     // length of wave
              lp  : Word ;     // pressure in 0.1 millibar
              lh  : Word ;     // humidity in percent
              lt  : Word ;     // temperature in 0.1 degree Kelvin
            end;
*)
unit UnForQLo ; // observations in Quick Look format
interface
var
  numqlh  : SmallInt ;  // number of headers
  numqlo  : SmallInt ;  // number of observations
procedure ToReadQuickLookFile ( st  : ShortString ; // fire with QL data
                                var  start  : extended ) ; // moment 
implementation
uses
  UnForDat , // procedure transdatetojd to julian day
  UnForTyq , // record TRecObs amd memory for observations
  unitsort ; // type treanum tintnum procedure ShellMethodForOrdering
var
  fl  : TextFile ; // with the observations
  no  : SmallInt ; // observations throw one passage
var
  secpartcur  : LongInt ; { current part of day in second }
{ to analyse header record }
procedure HeaderRecord ( stmp  : ShortString ) ;
var
  e  : integer ;  { value for error in Val procedure }
  y  : integer ;  { year current }
  d  : integer ;  { day from the start of year }
  t  : extended ; { for current date in modified julian day }
  s  : shortstring ; { current substring from our string }
begin
  { sat ident 1-7 , year 8-9 , day of year 10-12 }
  s:=Copy(stmp,1,7);  { number of satellite column 1-7 }
  Val(s,recobs.ns,e); { number of satellite to record }
  s:=Copy(stmp,8,2);  { year column 8-9 }
  Val(s,y,e);         { for year }
  If  e <> 0  Then  Exit ; { error }
  If  y < 71  Then  y:=2000+y  Else  y:=1900+y; { with century }
  TransDateToJD(1,1,y,t); { 1 january of current year UnForDat }
  s:=Copy(stmp,10,3); { day of year column 10-12 }
  Val(s,d,e);         { for day of current year }
  If  e <> 0  Then  Exit ; { error }
  recobs.dm:=t+d-1.0; { for day of year in julian day }
  { for station number column 13-16 }
  s:=Copy(stmp,13,4); { station number column 13-16 }
  Val(s,recobs.no,e); { station number to our record }
  If  e <> 0  Then  Exit ; { error }
  { for wave length column 21-24 in our string }
  s:=Copy(stmp,21,4); { wave length in 0.1 nanometer }
  Val(s,recobs.lw,e); { for wave length in 0.1 nanometer }
  secpartcur:=0;      { initial value to prevent jump in second }
  Inc(numqlh);        { number of headers }
end;
{ to analyse current record and to write to binary file }
procedure InformRecord ( stmp  : ShortString ) ;
var
  e  : integer ;  { value for error in Val procedure }
  l  : LongInt ;  { current integer }
  d  : extended ; { for second from start day variable }
  s  : shortstring ; { current substring from our string }
begin
  { string from quick look data with observations }
  s:=Copy(stmp,1,6); { time of day the first part column 1-6 }
  Val(s,l,e);        { 0.1 of integer part of second }
  If  e <> 0  Then  Exit ;
  If  l < secpartcur
    Then             { it is jump in second throw midnight }
      l:=864000+l;   { to prevent this siple jump }
  secpartcur:=l;     { new integer value to prevent jump }
  d:=1.0e-1*l; { to second }
  s:=Copy(stmp,7,6); { time of day the second part }
  Val(s,l,e);
  If  e <> 0  Then  Exit ;
  recobs.sp:=d+1.0e-7*l; { to second of time part of day }
  s:=Copy(stmp,13,6); { two-way time of flight the first part in picosecond }
  Val(s,l,e);
  If  e <> 0  Then  Exit ;
  d:=1.0e-6*l; { to second }
  s:=Copy(stmp,19,6); { two-way time of flight the second part }
  Val(s,l,e);
  If  e <> 0  Then  Exit ;
  recobs.ds:=d+1.0e-12*l; { delay two way in second of time }
  { from current record meteorologic data }
  s:=Copy(stmp,32,5);
  Val(s,recobs.lp,e); { pressure in 0.1 millibar }
  s:=Copy(stmp,37,4);
  Val(s,recobs.lt,e); { temperature on 0.1 degree Kelvin }
  s:=Copy(stmp,41,3);
  Val(s,recobs.lh,e); { relative humidity in percentage }
  Inc(numqlo); // number of observations
  Inc(no);     // throw passage
  numobs:=numobs+1; // count for observations from UnForTyq
  if  numobs > maxobs  then  numobs:=maxobs; // global UnForTyq
  arrobs^[numobs]:=recobs; // the next record to memory
end;
{ to analyse current string
  to find string 99999 b=False
  b=True to find header record }
procedure ForCurString ( s  : ShortString ;  var  b  : Boolean ) ;
var
  n  : Byte ;
begin
  if  Length(s) < 5  then  Exit ;  // too small string
  if  b     // current string
    then    // is the header string
      begin // for the header string
        HeaderRecord(s); // to analyse header record
        b:= not b; // the next strings
        Exit;      // may be 99999 or information string
      end;
  n:=Pos('99999',s); // may be the first record
  if  n = 1   // there is the first string 99999
    then      // True to analyse the next string
      begin
        no:=0;  // observations throw passage
        b:=True // as the header string
      end
    else      // to analyse the current record
      InformRecord(s); // and to write to memory
end;
{ to read record by record from file }
procedure ToReadQLFile ;
var
  a  : Byte ;
  c  : Char ;
  s  : ShortString ;
  b  : Boolean ;
begin
  b:=False;   // for the record 99999
  s:='';      // nullo string
repeat        // to read symbol by symbol
  Read(fl,c); // the next symbol from the file
  a:=Ord(c);  // ordinal position of character c
  if  ( a <> $0A ) and ( a <> $0D )  // may be $0A without $0D
    then      // the current symbol
      s:=s+c  // the next symbol to the our string
    else      // $0A $0D for the end of string
      begin   // this is the end of current string
        ForCurString(s,b);  // to analyse string
        s:='';      // nullo string for new record
      end;
until  EOF(fl) ;    // may be end of file
end;
{ to order array with observations with time
  by using shell method from unitsort ShellMethodForOrdering }
procedure ToSortQLFile ( var  start  : extended ) ; { shell method }
var
  f  : file of trecobs ; // UnForTyq type
  i  : longint ; // simple count
  k  : tintnum ; // type unitsort longint  array
  t  : treanum ; // type unitsort extended array
begin
  for i:=1 to numobs do // observation
    begin               // by observation
      k[i]:=i;          // initial number in list
      t[i]:=arrobs^[i].dm+arrobs^[i].sp/86400.0; // moment in day
    end;                // to order with time
  i:=numobs;            // numobs observations
  ShellMethodForOrdering(i,t,k); // unitsort procedure result in k
  { to write data to temporary binary file }
  AssignFile(f,'binqlo.tmp'); // temporary file
  ReWrite(f);   // to open to write
  i:=0;             // start value for count
repeat              // for all observations
  i:=i+1;           // to the next record
  write(f,arrobs^[k[i]]); // to write current record after ordering
until  i = numobs ; // for all observations
  CloseFile(f); // to close for the next action
  start:=arrobs^[k[1]].dm; // start moment in julian day
end;
{ to read and analyse observations in Quick Look format }
procedure ToReadQuickLookFile ( st  : ShortString ; // fire with QL data
                                var  start  : extended ) ; // moment 
begin                         { result initial moment to start process }
  ToGetObsMem ;        // from Unit UnForTyq  memory for observations
  AssignFile(fl,st);   // file with the observations
  ReSet(fl);           // to open file
  numqlh:=0;           // initial nullo headres
  numqlo:=0;           // initial nullo observations
  no:=0;               // initial nullo
  ToReadQLFile ;       // to read and analyse file
  CloseFile(fl);       // to close file
  ToSortQLFile(start); // by using shell method unitsort
  ToFreeObsMem ;       // from Unit UnForTyq memory for observations
end;
end.
