
 UNIT UnForSit ; { for greenwich sidereal time functions }

 INTERFACE

 Uses
   UnConTyp ; { for const and TMatr33 type }

 Function ToGetGrMSTime ( TinUTC , DeltaUT : Extended ) : Extended ;

 Function ToGetGrASTime ( TinUTC , DeltaUT : Extended ) : Extended ;

 IMPLEMENTATION

 Uses
   UnForTim , { for procedure TransUTCtoTDT }
   UnVakNup ; { for function ToGetEpsiloM procedure ClcNutPos }

 { ToGetGrMSTime is a function
   to obtain Greenwich Mean Sidereal Time in radians .
      IAU 1976  for the Greenwich Mean Sidereal Time .
      TinUTC in UTC and close to UT1 }

 Function ToGetGrMSTime( TinUTC , DeltaUT : Extended ) : Extended ;
 Var
   t,aj,ajj,ajs,s,s0,r,freq : Extended;
   SidTime  : Extended ;
   i : Integer;
 Begin
   t:=TinUTC+DeltaUT/86400.0;
   aj:=Trunc(t); s:=t-aj;
   ajj:=aj-51544.5; ajs:=ajj/36525.0;
   s0:=1.753368559233266e0+(628.3319706888409e0
	+(6.770713944903336e-06-4.508767234318685e-10*ajs)*ajs)*ajs;
   freq:=1.002737909350795e0
	+(5.900575455674703e-11-5.893984333409384e-15*ajs)*ajs;
   s0:=s0+freq*s*Pi22; r:=s0/Pi22; i:=Trunc(r);
   SidTime:=s0-Pi22*i;
   if  SidTime < 0  then  SidTime:=SidTime+Pi22;
   ToGetGrMSTime:=SidTime;
 End;

 { for true sidereal time }

 Function ToGetGrASTime ( TinUTC , DeltaUT : Extended ) : Extended ;
 Var
   t,td,du,dt,dp,de,eps  : Extended ;
 Begin
   t:=TinUTC;    { in mjd }
   du:=DeltaUT;  { in second of time }
   TransUTCtoTDT(t,td,dt); { from UnForTim to terrestrial time }
   eps:=ToGetEpsiloM(td); { function from UnVakNup epsilon in radian }
   ClcNutPos(td,dp,de);   { from UnVakNup nutation parameters in radian}
   ToGetGrASTime:=ToGetGrMSTime(t,du)+dp*Cos(eps);
 End;

 END.
