unit UnForTyq ; // types and variables for the observations
interface
uses
  UnConTyp , // type TVect3 for vector
  UnForTyp ; // type TDatPole for Earth rotation parameters
const
  maxobs  = 17177 ; // maximum number of observations in memory
type
  TRecObs = record             // simple record for observations
              ns  : LongInt ;  // number of the satellite
              no  : SmallInt ; // number of the observatory
              { moment in UTC  of firing on the observatory }
              dm  : extended ; // day as julian day
              sp  : Extended ; // part of day second of time
              ds  : Extended ; // time of delay two way in second
              lw  : Word ;     // length of wave in Angstrem 10^(-10)m
              lp  : Word ;     // pressure in 0.1 millibar
              lh  : Word ;     // humidity in percent
              lt  : Word ;     // temperature in 0.1 degree Kelvin
            end;
  TRecObr = record
              ns  : LongInt ;  // number of the satellite
              no  : SmallInt ; // number of the observatory
              dd  : extended ; // start day as julian day
              sd  : Extended ; // time from start day in second
              ro  : extended ; // laser range for dd+sd/86400 km
              wv  : extended ; // length of wave in mikrometer 10^(-6)m
              pr  : extended ; // athmospheric pressure in millibar
              hp  : extended ; // athmospheric humidity in percent
              tk  : extended ; // temperature in degree of Kelvin
              oz  : tvect3 ;   // station position Earth system UnConTyp
              zp  : extended ; // site geodetic latitude radian
              zl  : extended ; // site geodetic longitude radian
              zh  : extended ; // site height in km
              ot  : tvect3 ;   // station correction due tides  UnConTyp
              zs  : tvarsix ;  // short periodic corrections to elements
              zu  : extended ; // delta(UTC) in day
              zx  : extended ; // x pole position in radian
              zy  : extended ; // y pole position in radian
            end;
type  // for the array
  TMemObs  = array[1..maxobs] of TRecObs ; // type for big array 
  TMemPtr  = ^TMemObs ; // pointer to the array with observations
var
  numobs   : Word ;     // current number of observation
  arrobs   : TMemPtr ;  // variable as pointer to the array of records
  recobs   : TRecObs ;  // current record with observation
  ObrFile  : file of TRecObr ; // binary file 'binqlos.tmp'
  recobr   : TRecObr ;  // current record observation plus station
procedure ToGetObsMem ;
procedure ToFreeObsMem ;
implementation
procedure ToGetObsMem ;
begin
  numobs:=0; // initial nullo number fo observations
  GetMem(arrobs,SizeOf(TMemObs));  // memory for array
end;
procedure ToFreeObsMem ;
begin
  FreeMem(arrobs,SizeOf(TMemObs)); // free memory
end;
end.
