
 { to obtain Earth rotation parameters
      from the known source by linear interpolation }

 UNIT UnGetERP ;

 INTERFACE

 Uses
   UnForTyp ; { type TDatPole utc dut xpo ypo dxe dye }

 Var
   ERPFile  : TextFile ;
   BooERP   : Boolean ;
   ERPOne   : TDatPole ; { type UnForTyp }
   ERPTwo   : TDatPole ; { the left and the right values }
   erpmint  : Extended ; { the first moment in ERPFile }  
   erpmaxt  : Extended ; { the last  moment in ERPFile }  

 procedure ToReadERPFile ( var  erp  : TDatPole ) ; { UnForTyp }

 Procedure ToOpenERPFile ;

 Procedure ToCloseERPFile ;

 Procedure ToObtainERParm ( ut  : Extended ;  Var  dp : TDatPole ) ;

 IMPLEMENTATION

 uses
   UnConTyp ; { const JD2000 }

 procedure ToReadERPFile ( var  erp  : TDatPole ) ; { UnForTyp }
 var
   n  : Word ;
   a  : Extended ;
   t  : Extended ;
 begin
   if  not BooERP  then  Exit ; { no file with Earth Rotation Parameters }
   if  not EOF(ERPFile)
     then
       begin
         ReadLn(ERPFile,n,        { solve identificator }
                        t,        { time in modified julian day }
                        erp.xpo,  { x_pole in arcsec }
                        erp.ypo,  { y_pole in arcsec }
                        erp.dut,  { dUT=UT1-UTC in second of time }
                        a,        { length of day no use }
                        erp.dxe,  { dPsi celestial pole offset longitude " }
                        erp.dye); { dEps celestial pole offset obliquity " }
         ReadLn(ERPFile);   { string with root mean square errors }
         erp.utc:=DifEpoch+t; { time in julian day const UnConTyp }
       end;
 end;

 Procedure ToOpenERPFile ;
 var
   d  : TDatPole ; { type UnForTyp }
 Begin
   IndTime:=1; { UTC scale global UnForTyp }
   BooERP:=FALSE;
   AssignFile(ERPFile,'eop8899.txt');
 {$I-}
   ReSet(ERPFile);
 {$I+}
   if  IOResult <> 0  then  Exit ; { no file with ERParameters }
   BooERP:=True; { our file exists }
   ToReadERPFile(ERPOne); { the first  record }
   erpmint:=ERPOne.utc; { the first moment in julian day global here }
   ToReadERPFile(ERPTwo); { the second record }
   while  ( not EOF(ERPFile)  { may be some moments that are closed }
            and ( Abs(ERPTwo.utc-ERPOne.utc) < 0.1 ) )  do
     ToReadERPFile(ERPTwo); { to the next line with other moment }
   while  not EOF(ERPFile)  do { to find end of file in ERPFile }
     ToReadERPFile(d); { line by line from the first to last record }
   erpmaxt:=d.utc;     { the last  moment in julian day global here }
   ReSet(ERPFile);     { to the first record in file }
 End;

 Procedure ToCloseERPFile ;
 Begin
   If  BooERP  Then  CloseFile(ERPFile) ;
 End;

 { ut is moment in julian day
   result is in variable dp of TDatPole type
   TDatPole.(utc,dut,xpo,ypo,dxe,dye)
   dut - UT1-UTC
   xpo - empiric value x TRS pole position
   ypo - empiric value y TRS pole position
   dxe - empiric value delta(X) for CRS pole
   dye - empiric value delta(Y) for CRS pole
   linear interpolation
   x=x_1+(x_2-x_1)*(t-t_1)/(t_2-t_1) }

 Procedure ToObtainERParm ( ut  : Extended ;  Var  dp : TDatPole ) ;
 var
   dt  : Extended ; { for the time interval }
 Begin           { initial nullo as the first }
   dp.utc:=ut;   { current moment for parameters }
   dp.dut:=0.0 ; { UT1-UTC }
   dp.xpo:=0.0 ; { empiric value x TRS pole position }
   dp.ypo:=0.0 ; { empiric value y TRS pole position }
   dp.dxe:=0.0 ; { empiric value delta(X) for CRS pole }
   dp.dye:=0.0 ; { empiric value delta(Y) for CRS pole }
   if  not BooERP  then  Exit ; { no file with our parameters }
   if  ( ut < erpmint ) or ( erpmaxt < ut )  { our limit for parameters }
     then  Exit ; { moment is out of our limit in time for ERParameters }
   { to read ERPfile if possible }
   if  ut < ERPOne.utc { to compare }
     then              { our moment with current moment }
       begin           { our moment is less then current moment }
         ReSet(ERPFile); { to the start of file }
         ToReadERPFile(ERPOne); { the first  record }
         ToReadERPFile(ERPTwo); { the second record }
         while  ( not EOF(ERPFile)  { may be some moments that are closed }
                  and ( Abs(ERPTwo.utc-ERPOne.utc) < 0.1 ) )  do
           ToReadERPFile(ERPTwo); { to the next line with other moment }
       end;
   { to read ERPfile if possible
     to find interval of time for our moment ut }
   while  ( not EOF(ERPFile)
            and ( ERPTwo.utc < ut ) )  do { to compare }
     begin   { with current value to read line by line }
       ERPOne:=ERPTwo; { to the next interval of time }
       ToReadERPFile(ERPTwo); { to the next line with other moment }
       while  ( not EOF(ERPFile)  { may be some moments that are closed }
                and ( Abs(ERPTwo.utc-ERPOne.utc) < 0.1 ) )  do
         ToReadERPFile(ERPTwo); { to the next line with other moment }
     end;
   if  ( (ERPOne.utc-1.0e-6) < ut )      { to compare our moment }
        and ( ut < (ERPTwo.utc+1.0e-6) )  { with the current values }
     then { current moment is in known interval }
       begin
         dt:=ERPTwo.utc-ERPOne.utc; { interval in day between two values }
         if  dt < 0.1  then  Exit ; { interval in day is too small }
         dt:=(ut-ERPOne.utc)/dt; { linear interpolation (t-t_1)/(t_2-t_1) }
         dp.dut:=ERPOne.dut+(ERPTwo.dut-ERPOne.dut)*dt; { x=x_1+(x_2-x_1) }
         dp.xpo:=ERPOne.xpo+(ERPTwo.xpo-ERPOne.xpo)*dt; {  *(t-t_1) }
         dp.ypo:=ERPOne.ypo+(ERPTwo.ypo-ERPOne.ypo)*dt; {  /(t_2-t_1) }
       end; { our current ERP values are obtained }
 End;

 END.
