
 { the procedures to help to define model and data }

 UNIT UnHarMod ;

 INTERFACE

 Const
   MaxPowHarm    =   70 ;
   MaxZonHarm    =   70 ;
   MaxHarmCoef   = 2701 ;
   MaxEilHarm    =   16 ;

 Type
   THarmCoef     = Record { numerical value of the coefficients }
                     nh  : Byte ;
                     mh  : Byte ;
                     ch  : Extended ;
                     sh  : Extended ;
                   End;

 Type                     { for Legandre polinoms }
   TPolLeg       = array[0..MaxZonHarm] of Extended ; 

 Var
   MaxIntHarm    : Byte ; { number of the harmonics to integrate }
   GemParmValue  : Array [1..4] of Extended ;
   ZonHarmValue  : Array [1..MaxZonHarm] of Extended ;
   AllHarmValue  : Array [1..MaxHarmCoef] of THarmCoef ;
   EilHarmValue  : Array [1..MaxZonHarm] of Extended ;

 Procedure ToObtainGemParmValues ( strharmod  : ShortString ) ;

 Procedure ToObtainZonHarmValues ( strharmod  : ShortString ) ;

 Procedure ToObtainEilHarmValues ;

 Procedure ToObtainAllHarmValues ( strharmod  : ShortString ) ;

 Procedure ToWriteAboutHarmModel ( strharmod  : ShortString ) ;

 Procedure ToLimitHarmNumber ( nf  : Byte ) ;

 Procedure ToGetTwoCoef ( n , m  : Byte ;
                          Var  nc  : Word ;
                          Var  c , s  : Extended ) ;

 IMPLEMENTATION

 Uses
   UnConTyp , { for GeoFM }
   UnForPrt , { for PrtFile }
   UnForFun ; { to work with complex numbers }

 { to obtain from the file with the name strharmod
   the values of the coefficients fm and r0 }

 Procedure ToObtainGemParmValues ( strharmod  : ShortString ) ;
 Var
   fg  : TextFile ;
   fm  : Extended ;
   re  : Extended ;
 Begin
   AssignFile(fg,strharmod);
   ReSet(fg);
   ReadLn(fg);
   ReadLn(fg);
   ReadLn(fg,re); { for Earth equatorial radius in m}
   ReadLn(fg,fm); { for geocentric gravitational const in m^3/s^2 }
   GemParmValue[1]:=1.0e-9*fm; { to km^3/s^2 }
   GemParmValue[2]:=1.0e-3*re; { to km }
   GeoFM:=GemParmValue[1];
   GeoR0:=GemParmValue[2];
   CloseFile(fg);
 End;

 { to obtain the values of the coefficients of the zonal harmonics }

 Procedure ToObtainZonHarmValues ( strharmod  : ShortString ) ;
 Var
   GemFile             : TextFile ;
   i,m,n               : Byte ;
   r,JHarm             : Extended ;
 Begin
   AssignFile(GemFile,strharmod);
   ReSet(GemFile);
   For n:=1 To MaxZonHarm Do  ZonHarmValue[n]:=0;
   For i:=1 To 9 Do  ReadLn(GemFile);
   n:=1;
   While  n < MaxZonHarm  Do
     Begin
       ReadLn(GemFile,n,m,JHarm);
       If  m = 0
         Then
           Begin
             r:=n; { to unnormalize the value of coefficients }
             ZonHarmValue[n]:=-1.0e-6*Sqrt(2*r+1)*JHarm;
           End;
     End;
   CloseFile(GemFile);
   MaxIntHarm:=n;
 End;

 { to obtain simple parameters  c , sigma , J'n
   of the intermediate generalised two fixed centre problem }

 Procedure ToObtainEilHarmValues ;
 Var
   n        : Byte ;
   q,r      : Extended ;
   x,y,a,b  : TCompValue ;
   xa,yb,c  : TCompValue ;
   rz       : Extended ;
   zj2,zj3  : Extended ;
   cj2,sig  : Extended ;
 Begin
   For n:=1 To MaxZonHarm Do  EilHarmValue[n]:=0.0; { initial nullo }
   rz:=GemParmValue[2];   { equatorial radius of the Earth r_0 }
   zj2:=ZonHarmValue[2];  { coefficient for the second zonal harmonic J_2 }
   zj3:=ZonHarmValue[3];  { coefficient for the third  zonal harmonic J_3 }
   r:=zj3/(2*zj2);        { (J_2)/(2*J_2) }
   cj2:=rz*Sqrt(zj2-Sqr(r)); { parameter c of intermediate potential }
   sig:=r*rz/cj2;       { parameter \sigma of intermediate potential }
   GemParmValue[3]:=cj2;  { parameter c of intermediate potential }
   GemParmValue[4]:=sig;  { parameter \sigma of intermediate potential }
   q:=cj2/rz;             { c/r_0 }
   r:=q;                  { (c/r_0)^1 }
   x.rval:=+1;    { complex x  = 1+i\sigma  i=\sqrt(-1) real(x) = +1 }
   x.ival:=+sig;  { imagine(x) = +\sigma }
   y.rval:=+1;    { complex y  = 1-i\sigma  real(y) - +1 }
   y.ival:=-sig;  { imagine(y) = -\sigma }
   a.rval:=+sig;  { complex a  = \sigma+i   real(a) = \sigma }
   a.ival:=+1;    { imagine(a) = +1 }
   b.rval:=+sig;  { complex b  = \sigma-i   real(b) = \sigma }
   b.ival:=-1;    { imagine(b) = -1 }
   MultComplex(x,a,xa); { from UnForFun }
   MultComplex(y,b,yb);
   x:=xa;
   y:=yb;
   For n:=1 To MaxEilHarm Do { value by value }
     Begin
       c.rval:=x.rval+y.rval;
       c.ival:=x.ival+y.ival;
       EilHarmValue[n]:=-0.5*r*c.rval; { current value }
       r:=r*q;                         { (c/r_0)^(n+1) }
       MultComplex(x,a,xa);
       MultComplex(y,b,yb);
       x:=xa;
       y:=yb;
     End;
 End;

 { without intermediate potential }

 Procedure ToChangeZonHarmValues ; { j_n = J'_n-J_n }
 Var
   n  : Byte ;
 Begin
   For n:=1 To MaxZonHarm Do
     ZonHarmValue[n]:=EilHarmValue[n]-ZonHarmValue[n];
 End;

 { to obtain the values of the coefficients
      of the zonal , tesseral and sectorial harmonics }

 Procedure ToObtainAllHarmValues ( strharmod  : ShortString ) ;
 Var
   GemFile   : TextFile ;
   i,k,m,n   : Byte ;
   NumH      : Word ;
   p,q,r,z   : Extended ;
   Cnm,Snm   : Extended ;
 Begin
   ToObtainGemParmValues(strharmod); { fM r_0 parameters }
   ToObtainZonHarmValues(strharmod); { J_n cofficients }
   ToObtainEilHarmValues ; { intermediate potential J'_n }
   ToChangeZonHarmValues ; { for j_n = J'_n - J_n coefficients }
   AssignFile(GemFile,strharmod);
   ReSet(GemFile);
   For NumH:=1 To MaxHarmCoef Do
     With  AllHarmValue[NumH]  Do
       Begin
         nh:=0;
         mh:=0;
         ch:=0.0;
         sh:=0.0;
       End;
   For i:=1 To 9 Do  ReadLn(GemFile);
   NumH:=0;
   n:=0;
   While  NOT EOF(GemFile)  Do
     Begin
       ReadLn(GemFile,n,m,Cnm,Snm);
       If  ( ( n <= MaxPowHarm ) AND ( m <= MaxPowHarm ) )
         Then { all coefficients zonal tesseral sectorial }
           Begin
             NumH:=NumH+1;
             p:=1;
             z:=n;
             r:=n-m;
             k:=2*m;
             For i:=1 To k Do  p:=p/(r+i);   { (n-m)!/(n+m)! }
             If  m = 0 { to unnormalize the value of coefficients }
               Then { zonal harmonics }
                 q:=Sqrt(2*z+1) { Sqrt(2n+1) }
               Else { other harmonics m > 0 }
                 q:=Sqrt(2*(2*z+1)*p); { Sqrt(2*(2*n+1)*(n-m)!/(n+m)!) }
             With  AllHarmValue[NumH]  Do
               Begin
                 nh:=n;
                 mh:=m;
                 ch:=+1.0e-6*q*Cnm;
                 sh:=+1.0e-6*q*Snm;
               End;
           End;
     End;
   CloseFile(GemFile);
   MaxIntHarm:=n;
 End;

 Procedure ToWriteAboutHarmModel ( strharmod  : ShortString ) ;
 Var
   nharm  : Word ;
 Begin
   WriteLn(PrtFile,'model from the file  ',strharmod);
   WriteLn(PrtFile,'geocentric gravitational constant',
                   GemParmValue[1]:15:5,'  km**3/s**2');
   WriteLn(PrtFile,'   equatorial radius of the Earth',
                   GemParmValue[2]:15:5,'  km');
   WriteLn(PrtFile,'parametr     c',GemParmValue[3]:25:18,'  km');
   WriteLn(PrtFile,'parametr sigma',GemParmValue[4]:25:18);
   WriteLn('coefficients for zonal harmonics');
   WriteLn(' ':9,'n',' ':10,'1e6*Jn');
   WriteLn(PrtFile,'coefficients for zonal harmonics');
   WriteLn(PrtFile,' ':9,'n',' ':10,'1e6*Jn');
   For nharm:=2 To MaxZonHarm Do
     If  nharm <= MaxEilHarm
       Then
         Begin
           WriteLn(nharm:10,1.0e6*ZonHarmValue[nharm]:20:10);
           WriteLn(PrtFile,nharm:10,1.0e6*ZonHarmValue[nharm]:20:10,
                                    1.0e6*EilHarmValue[nharm]:30:18);
         End;
   WriteLn('coefficients for tesseral and sectorial harmonics');
   WriteLn('    n    m',' ':10,'1e6*Cnm   ',' ':10,'1e6*Snm');
   WriteLn(PrtFile,'coefficients for tesseral and sectorial harmonics');
   WriteLn(PrtFile,'    n    m',' ':20,'1e6*Cnm   ',' ':20,'1e6*Snm');
   For nharm:=2 To MaxHarmCoef Do
     With  AllHarmValue[nharm]  Do
       If  ( ( ( 0 < nh ) AND ( nh < 11 ) )
             AND ( ( 0 < mh ) AND ( mh < 11 ) ) )
         Then
           Begin
             WriteLn(nh:5,mh:5,
                     1.0e6*ch:30:15,1.0e6*sh:30:15);
             WriteLn(PrtFile,nh:5,mh:5,
                             1.0e6*ch:30:25,1.0e6*sh:30:25);
           End;
 End;

 Procedure ToLimitHarmNumber ( nf  : Byte ) ;
 Begin
   If  nf > 1
     Then
       MaxIntHarm:=nf
     Else
       MaxIntHarm:=0;
 End;

 { to get current values
   of harmonic coefficients Cnm , Snm
      from AllHarmValue[NumHarmCoef] }

 Procedure ToGetTwoCoef ( n , m  : Byte ;
                          Var  nc  : Word ;
                          Var  c , s  : Extended ) ;
 Var
   nx  : Byte ;
   mx  : Byte ;
 Begin
   Repeat
     nc:=nc+1;
     nx:=AllHarmValue[nc].nh;
     mx:=AllHarmValue[nc].mh;
   Until ( ( ( nx = n ) AND ( mx = m ) ) OR ( nx = 0 ) ) ;
   c:=AllHarmValue[nc].ch;
   s:=AllHarmValue[nc].sh;
 End;

 END.
