
 { for partial derivation
   to data processing and to pseudoinverse
   six nonsingular elements TVarSix from UnConTyp :
   a l+g+h e*cos(g+h) e*sin(g+h) \delta*cos(h) \delta*sin(h) }

 UNIT UnMatDer ;

 INTERFACE

 Uses
   UnConTyp , { const types }
   UnForTyp ; { type TVarSix global variables }

 { for pseudoinversion }
 Const
   MatrN   = 8 ; { + 6 - nonsingular elements }
   MatrM   = 8 ; { + 6 - nonsingular elements
                   + 1 - reflection coefficient c_r
                   + 1 - empiric acceleration coefficient c_t }

 Type
   TypeDimN    = Array [1..MatrN] of Extended ;
   TypeDimM    = Array [1..MatrM] of Extended ;
   TypeDimNM   = Array [1..MatrN,1..MatrM] of Extended ;
   TypeDimMN   = Array [1..MatrM,1..MatrN] of Extended ;
   TypeDimInt  = Array [1..MatrM] of Integer ;

 Type
   TRangeDer   = Record
                   day , secp  : extended ; { start day second from }
                   numbersite  : smallint ; { to identify the station }
                   RoEtalon    : extended ; { range value in km Obs }
                   RoResidual  : extended ; { current residual in km }
                   RoNulloVar  : extended ; { range nullo variant in km }
                   RoDerivats  : TypeDimM ; { range derivations in km }
                 End;

 Const
   DifParam   : TypeDimM  =  { 8 variables }
                    ( 1.0e-3 , { for semimajor axis a in km }
                      1.0e-5 , { for l+g+h in radian }
                      1.0e-6 , { for e*cos(g+h) }
                      1.0e-6 , { for e*sin(g+h) }
                      1.0e-5 , { for \delta*cos(h) }
                      1.0e-5 , { for \delta*sin(h) }
                      1.0e-3 , { for pressure coef. k_r*A_r/m_r m^2/kg }
                      1.0e-8 ) ; { for empiric acceleration rev./day^2 }

 Var
   DerFile    : File of TRangeDer ;
   DerRec     : TRangeDer ;
   NumPer     : LongInt ;   { count in file }
   numter     : Byte ;      { number of iterations }
   tresigma   : extended ;  { a value to exclude points }

 Procedure ToObtainDerivations ( eb  : TVarSix ) ;  { initial elements }

 Procedure ClcVariablesCorrections ( Var  DifPar  : TypeDimM ) ;

 Procedure ToObtainResiduals ( eb  : TVarSix ) ;  { initial elements }

 IMPLEMENTATION

 Uses
   UnForPrt , { prtfile text file }
   UnForTyq , { global record recobr and binary file ObrFile }
   UnTypVar , { global variables step in day for integration fulstp }
   UnForFun , { proc MultMatrVec }
   UnForSop , { global var satarama in m^2/kg satvalar satvalmr }
   UnEverGo , { proc EverGratorFord global fulstp form UnTypVar }
   UnVashad , { proc EverGratorShad }
   UnVakSha , { global numshadow shadowone[..] shadowtwo[..] }
   UnForStr , { function FloatToSingle }
   UnFotrua , { proc ToObtainTruEquT with ERP values }
   UnTusRoC , { function ToGetTopoRange for current recobr }
   UnMatInv ; { PseudoInVerse }

 Var
   corefl : Extended ;  { refl.coef. satarama to keep value in memory }
   coremp : Extended ;  { emp.accel. satempac to keep value in memory }

 Procedure ToVariateOneParam ( nump  : Byte ;
                               elmf  : TVarSix ; { type UnForTyp }
                               Var  elm  : TVarSix ) ;
 Begin
   elm:=elmf;
   satarama:=corefl; { this is a value that was keep global UnForSop }
   satempac:=coremp; { this is a value that was keep global UnForSop }
   Case  nump  Of
     0     :  Exit ; { no variations for nullo variant }
     1..6  :  elm[nump]:=elmf[nump]+DifParam[nump]; { to elements }
     7     :  satarama:=corefl+DifParam[nump]; { reflection coef. }
     8     :  satempac:=coremp+DifParam[nump]; { emp.accel. coef. }
   End; { case }
 End;

 Procedure ToReWriteDerFile ( nd  : Byte ;      { number variant }
                              tc  : Extended ;  { moment in second }
                              ez  : TVarSix ) ; { nonsingular elements }
 Var
   i  : byte ;     { count }
   r  : extended ; { range in km }
   e  : tvarsix ;  { osculating nonsingular elements }
 Begin
   for i:=1 to 6 do            { for six nonsingular elements }
     e[i]:=ez[i]+recobr.zs[i]; { to add short periodic terms }
   r:=ToGetTopoRange(tc,e);    { UnTusRoC by using global recobr }
   If  nd = 0
     Then { nullo variant }
       With  DerRec  Do
         Begin
           RoNulloVar:=r; { topocentric range Calculated in km }
           RoResidual:=1.0e3*(RoEtalon-RoNulloVar); { in metr O-C }
           if  ( ( numper mod 100 ) = 0 )  then
             WriteLn(numter:3,nd:3,numbersite:8,
                     tc/86400:20:12,RoResidual:20:3,' m');
         End
     Else { with variation }
       With  DerRec  Do
         Begin
           RoDerivats[nd]:=1.0e3*(r-RoNulloVar); { in metr }
           if  ( ( numper mod 100 ) = 0 )  then
             WriteLn(numter:3,nd:3,numbersite:8,
                     tc/86400:20:12,RoDerivats[nd]:20:3,' m');
         End;
   Seek(DerFile,NumPer);
   Write(DerFile,DerRec);
 End;

 { to calculate step by step isochronic derivations
      ds   step in second for numerical integration }

 Procedure ToObtainDerivations ( eb  : TVarSix ) ;  { initial elements }
 Var
   dstep  : Extended ; { step to integrate }
   NumD   : Byte ;
   MaxD   : Byte ;
   tc,zt  : Extended ; { time in second }
   ed,ee  : TVarSix ;  { type from UnForTyp nonsingular elements }
   ev,ez  : TVarSix ;  { nonsingular elements }
   td,te  : Extended ;
 Begin
   dstep:=86400*fulstp; { global variable UnTypVar step in day to second }
   corefl:=satarama;    { global var UnForSop corefl to keep in memory }
   coremp:=satempac; { global var UnForSop coremp to keep in memory }
   if  eb[1] > 14000.0  { semimajor axis in kilometer }
     then  MaxD:=7      { without empiric acceleration }
     else  MaxD:=8; { 6 nonsingular elements +1 refl.coef. +1 emp.acc. }
   For NumD:=0 To MaxD Do
     Begin  { with variation for one current parametr }
       ToVariateOneParam(NumD,eb,ev); { elements eb are fixed }
       td:=0.0; { initial moment in second from IntTime.begday }
       ed:=ev;  { initial elements values }
       te:=td;  { initial moment in second from IntTime.begday }
       ee:=ed;  { to start to integrate elements }
       EvorInit ;      { from UnEverGo to start EverGratorFord }
       EvorCoefNullo ; { from UnEverGo to start EverGratorFord }
       ReSet(DerFile); { UnMatDer file is assigned in UnTusDoi }
       ReSet(ObrFile); { UnForTyq file is assigned in UnTusDoi }
       NumPer:=-1;
       While  NOT EOF(DerFile)  Do
         Begin
           Read(DerFile,DerRec); { global variables from here }
           { to read file with observations }
           Read(ObrFile,recobr); { global unfortyq observations }
           tc:=DerRec.secp; { moment in second  from IntTime.begday }
           NumPer:=NumPer+1; { number of record to read }
           While  ( te < (tc-1.0e-8) )  Do { to integrate from UnEverGo }
//             EverGratorFord(dstep,td,ed,te,ee); { from UnEverGo }
             EverGratorShad(dstep,td,ed,te,ee); { from UnVashad }
           zt:=tc-td; { interval in second }
           VarFromPolinom(zt,ed,ez); { UnEverGo result in interval }
           { to obtain all matrix for current observations
             matrix TruEquT to change from intermed.equa to terrestrial }
           ToObtainTruEquT(recobr.sd,TerEquT,TruEquT); { UnFotrua UnForTyp }
           ToReWriteDerFile(NumD,tc,ez);
         End;
       CloseFile(ObrFile);
       CloseFile(DerFile);
     End;
   satarama:=corefl; { global var UnForSop corefl to keep in memory }
   satempac:=coremp; { global var UnForSop coremp to keep in memory }
 End;

 Procedure ToReCalcResiduals ( MMatr  : Integer ;
                               Var  DMin , DMax , DifS  : Extended ;
                               Var  DifPar  : TypeDimM ) ;
 Var
   j     : Byte ;
   NumP  : Integer ;
   DifA  : Extended ;
 Begin
   WriteLn(prtfile);
   Write(prtfile,' ':5); { simple title for table }
   Write(prtfile,' ':46,'r (O-C) metr');
   WriteLn(prtfile);
   Write(prtfile,' ':5);
   WriteLn(prtfile,' ':26,'before':20,'after':20);
   WriteLn(prtfile);
   NumP:=0;
   DifS:=0;
   DMax:=-1.0e6;
   DMin:=+1.0e6;
   While  NOT EOF(DerFile)  Do
     Begin
       Read(DerFile,DerRec);
       With  DerRec  Do
         Begin
           NumP:=NumP+1;
           Write(prtfile,NumP:5);
           DifA:=RoResidual; { in m }
           For j:=1 To MMatr Do
             If  DifParam[j] > 1.0e-13  Then
               DifA:=DifA-RoDerivats[j]*DifPar[j]/DifParam[j];
           Write(prtfile,numbersite:8,
                         secp/86400.0:18:12, { in day }
                         RoResidual:20:3,DifA:20:3,' m');
           if  ( Abs(DifA) < tresigma )
             then
               begin
                 DifS:=DifS+Sqr(DifA); { only for alpha delta }
                 If  DifA < DMin  Then  DMin:=DifA;
                 If  DifA > DMax  Then  DMax:=DifA;
                 write(prtfile,'   ');
               end
             else
               write(prtfile,' * ');
         End;
       WriteLn(prtfile);
     End;
 End;

 { to obtain corrections of the parameters by the least squares method }

 Procedure ClcVariablesCorrections ( Var  DifPar  : TypeDimM ) ;
 Var
   i,j                    : Byte ;
   NumP,
   MatrRank,NMatr,MMatr   : Integer ;
   q,DMax,DMin,
   DifS,Sigma,Epsilon     : Extended ;
   MatrVect               : TypeDimInt ;
   AMatr                  : TypeDimNM ;
   BMatr                  : TypeDimMN ;
   DifVect                : TypeDimN ;
   DParam,SigmaP          : TypeDimM ;
 Begin
   WriteLn;
   WriteLn('matrix inverse');
   NMatr:=MatrN;
   MMatr:=MatrM;
   Epsilon:=1.0e-13;
   For i:=1 To NMatr Do
     Begin
       DifVect[i]:=0.0;
       For j:=1 To MMatr Do
         Begin
           AMatr[i,j]:=0.0;
           BMatr[j,i]:=0.0;
         End;
     End;
   ReSet(DerFile);
   NumP:=0;
   DifS:=0;
   While  NOT EOF(DerFile)  Do
     Begin
       Read(DerFile,DerRec);
       With  DerRec  Do
         Begin
           NumP:=NumP+1;
           if  ( Abs(RoResidual) < tresigma )  then
             begin
               DifS:=DifS+Sqr(RoResidual); { in m }
               For i:=1 To NMatr Do
                 Begin
                   DifVect[i]:=DifVect[i]
                               +RoResidual*RoDerivats[i];
                   For j:=i To MMatr Do
                     AMatr[i,j]:=AMatr[i,j]
                                 +RoDerivats[i]*RoDerivats[j];
                 End;
             end; { if Abs(ro) < tresigma }
         End; { with DerRec }
     End;
   For i:=1 To NMatr Do
     For j:=i To MMatr Do
       AMatr[j,i]:=AMatr[i,j];
   WriteLn('number of points',NumP:6);
   WriteLn(prtfile,'number of points',NumP:6);
   PseudoInverse(AMatr,NMatr,MMatr,Epsilon,MatrRank,MatrVect,BMatr);
   WriteLn('rank of matrix',MatrRank:5);
   Write('on variables :');
   For i:=1 To MMatr Do  Write(MatrVect[i]:4);
   WriteLn;
   WriteLn(prtfile,'rank of matrix',MatrRank:5);
   Write(prtfile,'on variables :');
   For i:=1 To MMatr Do  Write(prtfile,MatrVect[i]:4);
   WriteLn(prtfile);
   q:=NumP;
   If  q < 1.0  Then  q:=1;
   Sigma:=Sqrt(DifS/q); { in metr }
   WriteLn('  RMS error in metr',Sigma:10:3);
   WriteLn;
   tresigma:=5.0*Sigma; { to exclude bad points }
   For j:=1 To MMatr Do
     Begin
       q:=0;
       For i:=1 To NMatr Do
         q:=q+BMatr[j,i]*DifVect[i];
       DifPar[j]:=q*DifParam[j];
     End;
   WriteLn(' corrections to nonsingular elements');
   WriteLn(DifPar[1]:20:10,
           DifPar[2]:20:10,'  a (km) l+g+h (radian)');
   WriteLn(DifPar[3]:20:10,
           DifPar[4]:20:10,'  e*cos(g+h)  e*sin(g+h)');
   WriteLn(DifPar[5]:20:10,
           DifPar[6]:20:10,'  \delta*cos(h) \delta*sin(h)');
   WriteLn(' correction  to reflection coef. k_r*A_midel/m_sat');
   WriteLn(FloatToSingle(DifPar[7],20):25,'  in m^2/kg');     { UnForStr }
   WriteLn(' correction  to empiric acceleration');
   WriteLn(FloatToSingle(DifPar[8],20):25,'  rev/day^2');     { UnForStr }
   WriteLn(prtfile,' corrections to nonsingular elements');
   WriteLn(prtfile,DifPar[1]:20:10,
                   DifPar[2]:20:10,'  a (km) l+g+h (radian)');
   WriteLn(prtfile,DifPar[3]:20:10,
                   DifPar[4]:20:10,'  e*cos(g+h)  e*sin(g+h)');
   WriteLn(prtfile,DifPar[5]:20:10,
                   DifPar[6]:20:10,'  \delta*cos(h) \delta*sin(h)');
   WriteLn(prtfile,' correction  to reflection coef. k_r*A_midel/m_sat');
   WriteLn(prtfile,FloatToSingle(DifPar[7],20):25,'  in m^2/kg'); { UnForStr }
   WriteLn(prtfile,' correction  to empiric acceleration');
   WriteLn(prtfile,FloatToSingle(DifPar[8],20):25,'  rev/day^2'); { UnForStr }
   { to recalculate residuals }
   ReSet(DerFile);
   ToReCalcResiduals(MMatr,DMin,DMax,DifS,DifPar);
   Close(DerFile);
   q:=NumP-MatrRank;
   If  q < 1.0  Then  q:=1.0;
   Sigma:=Sqrt(DifS/q);
   For j:=1 To MMatr Do
     Begin
       DParam[j]:=DifPar[j];
       SigmaP[j]:=Sigma*Sqrt(Abs(BMatr[j,j]))*DifParam[j];
     End;
   WriteLn;
   WriteLn('  RMS error in metr',Sigma:10:3);
   WriteLn('  residuals ',
           '   from',DMin:10:3,
           '     to',DMax:10:3,'  in metr');
   WriteLn(prtfile);
   WriteLn(prtfile,'  RMS error in metr',Sigma:10:3);
   WriteLn(prtfile,'  residuals ',
           '   from',DMin:10:3,
           '     to',DMax:10:3,'  in metr');
 End;

 Procedure ToPrintResiduals ( tc  : Extended ;  { moment in second }
                              ez  : TVarSix ) ; { nonsingular elements }
 Var
   i  : byte ;     { count }
   r  : extended ; { range in km }
   e  : tvarsix ;  { osculating nonsingular elements }
 Begin
   for i:=1 to 6 do            { for six nonsingular elements }
     e[i]:=ez[i]+recobr.zs[i]; { to add short periodic terms }
   r:=ToGetTopoRange(tc,e);    { UnTusRoC by using global recobr }
   with  DerRec  do
     begin
       RoResidual:=1.0e3*(RoEtalon-r); { in metr O-C }
       WriteLn(NumPer:6,numbersite:8,
               tc/86400:20:12,RoResidual:20:3,' m');
       WriteLn(prtfile,NumPer:6,numbersite:8,
                       tc/86400:20:12,RoResidual:20:3,' m');
     end;
 End;

 { to calculate step by step to prognoz motion
      fulstp    step in second for numerical integration }

 Procedure ToObtainResiduals ( eb  : TVarSix ) ;  { initial elements }
 Var
   dstep  : Extended ; { step to integrate }
   NumD   : Byte ;
   MaxD   : Byte ;
   tc,zt  : Extended ; { time in second }
   ed,ee  : TVarSix ;  { type from UnForTyp nonsingular elements }
   ev,ez  : TVarSix ;  { nonsingular elements }
   td,te  : Extended ;
 Begin
   dstep:=86400*fulstp; { global variable UnTypVar step in day to second }
   corefl:=satarama;    { global var UnForSop corefl to keep in memory }
   coremp:=satempac; { global var UnForSop coremp to keep in memory }
   MaxD:=0; { only for residuals }
   For NumD:=0 To MaxD Do { here nullo only for prognoz }
     Begin  { with variation for one current parametr }
       ToVariateOneParam(NumD,eb,ev); { elements eb are fixed }
       td:=0.0; { initial moment in second from IntTime.begday }
       ed:=ev;  { initial elements values }
       te:=td;  { initial moment in second from IntTime.begday }
       ee:=ed;  { to start to integrate elements }
       EvorInit ;      { from UnEverGo to start EverGratorFord }
       EvorCoefNullo ; { from UnEverGo to start EverGratorFord }
       ReSet(DerFile); { UnMatDer file is assigned in UnTusDoi }
       ReSet(ObrFile); { UnForTyq file is assigned in UnTusDoi }
       NumPer:=-1;
       While  NOT EOF(DerFile)  Do
         Begin
           Read(DerFile,DerRec); { global variables from here }
           { to read file with observations }
           Read(ObrFile,recobr); { global unfortyq observations }
           tc:=DerRec.secp; { moment in second  from IntTime.begday }
           NumPer:=NumPer+1; { number of record to read }
           While  ( te < (tc-1.0e-8) )  Do { to integrate from UnEverGo }
//             EverGratorFord(dstep,td,ed,te,ee); { from UnEverGo }
             EverGratorShad(dstep,td,ed,te,ee); { from UnVashad }
           zt:=tc-td; { interval in second }
           VarFromPolinom(zt,ed,ez); { UnEverGo result in interval }
           { to obtain all matrix for current observations
             matrix TruEquT to change from true equator to terrestrial }
           ToObtainTruEquT(recobr.sd,TerEquT,TruEquT); { UnFotrua UnForTyp }
           ToPrintResiduals(tc,ez);
         End;
       CloseFile(ObrFile);
       CloseFile(DerFile);
     End;
   satarama:=corefl; { global var UnForSop corefl to keep in memory }
   satempac:=coremp; { global var UnForSop coremp to keep in memory }
 End;

 END.
