
 { to calculate Q(t) matrix with the use value for X Y and s(t)
                R(-theta) matrix with the use linear formulae for theta }

 UNIT UnQM2000 ;

 INTERFACE

 Uses
   UnConTyp ; { for types and constants }

 Procedure QMatr2000 ( DateJulian  : Extended ;
                       DeltaXEmp   : Extended ; { delta(X) , delta(Y) }
                       DeltaYEmp   : Extended ; { empiric value in arcsec }
                       Var  QMatr  : TMatr33  ) ;

 Procedure QRMatr2000 ( DateJulian   : Extended ;
                        DateJulUT1   : Extended ;
                        DeltaXEmp    : Extended ;
                        DeltaYEmp    : Extended ;
                        Var  QRMatr  : TMatr33  ) ;

 Procedure QMapp2000 ( DateJulian  : Extended ; { moment of time }
                       DeltaXEmp   : Extended ; { delta(X) , delta(Y) }
                       DeltaYEmp   : Extended ; { empiric value in arcsec }
                       Var  QMatr  : TMatr33  ) ;

 Procedure QRMapp2000 ( DateJulian   : Extended ;
                        DateJulUT1   : Extended ;
                        DeltaXEmp    : Extended ;
                        DeltaYEmp    : Extended ;
                        Var  QRMatr  : TMatr33  ) ;

 IMPLEMENTATION

 Uses
   UnXY2000 , { procedure FormulaeXY }
   Un2000_S , { function ToGetCeoPos }
   Un2000_r , { for procedure RMatr2000 }
   UnXYSApp , { for XYSFromApprox } 
   UnForFun ; { for simple functions  ForRotMatr ToMultMatr }

 Procedure FormulaXYS ( d  : Extended ;
                        Var  x , y , s  : Extended ) ;
 Begin
   FormulaeXY(d,x,y); { XCip , YCip in radian , pole position UnXY2000 }
   s:=ToGetCEOPos(d,x,y); { s(t) CEO position in ArcSec from Un2000_S }
   x:=RadSec*x; { X position to ArcSec }
   y:=RadSec*y; { Y position to ArcSec }
 End;

 { to calculate Q(t) matrix 
   Q(t)=q(X,Y)*z(+s) with the use value for X Y and s(t) }

 Procedure QMatr2000 ( DateJulian  : Extended ; { moment of time }
                       DeltaXEmp   : Extended ; { delta(X) , delta(Y) }
                       DeltaYEmp   : Extended ; { empiric value in arcsec }
                       Var  QMatr  : TMatr33  ) ;
 Var
   d,s  : Extended ;
   x,y  : Extended ;
   q,z  : TMatr33 ;
   x2,y2,aa,cd  : Extended ;
 Begin
   d:=DateJulian; { date in Julian Day }
   FormulaXYS(d,x,y,s); { XCip , YCip , s(t) in ArcSec }
   x:=SecRad*(x+DeltaXEmp); { with empirical correction }
   y:=SecRad*(y+DeltaYEmp); { with empirical correction }
   x2:=Sqr(x); { x*x }
   y2:=Sqr(y); { y*y }
   cd:=Sqrt(1-(x2+y2)); { z=cos(d) , x y z is unit vector }
   aa:=1.0/(1+cd);
   q[1,1]:=+1-aa*x2;
   q[1,2]:=-aa*x*y;
   q[1,3]:=+x;
   q[2,1]:=-aa*x*y;
   q[2,2]:=+1-aa*y2;
   q[2,3]:=+y;
   q[3,1]:=-x;
   q[3,2]:=-y;
   q[3,3]:=+1-aa*(x2+y2); { this is part of Q(t) matrix }
   s:=SecRad*s; { a value for s(t) CEO position to radian }
   ForRotMatr(3,s,z); { to rotate about z-axis on the angle s(t) in radian }
   ToMultMatr(q,z,QMatr); { full Q(t) matrix unit UnForFun }
 End;

 { to multiplicate Q(t) and R(t) matrixes
   from the equator of the CIP to the fixed equator GCRS }

 Procedure QRMatr2000 ( DateJulian   : Extended ;
                        DateJulUT1   : Extended ;
                        DeltaXEmp    : Extended ;
                        DeltaYEmp    : Extended ;
                        Var  QRMatr  : TMatr33  ) ;
 Var
   QMatr,RMatr  : TMatr33 ;
 Begin
   RMatr2000(DateJulUT1,RMatr); { unit Un2000_r }
   QMatr2000(DateJulian,DeltaXEmp,DeltaYEmp,QMatr);
   ToMultMatr(QMatr,RMatr,QRMatr); { full Q(t)*R(-theta) matrix }
 End;

 { to calculate Q(t) matrix 
   Q(t)=q(X,Y)*z(+s) with the use value for X Y and s(t)
   in this procedure binary file parmaps.pat
   with approximations coefficients is used }

 Procedure QMapp2000 ( DateJulian  : Extended ; { moment of time }
                       DeltaXEmp   : Extended ; { delta(X) , delta(Y) }
                       DeltaYEmp   : Extended ; { empiric value in arcsec }
                       Var  QMatr  : TMatr33  ) ;
 Var
   n  : LongInt ;
   d,s  : Extended ;
   p,e  : Extended ;
   x,y  : Extended ;
   q,z  : TMatr33 ;
   x2,y2,aa,cd  : Extended ;
 Begin
   d:=DateJulian; { date in Julian Day }
   XYSFromApprox(d,n,p,e,x,y,s); { unit UnXYSApp parameters in ArcSec }
   If  n < 0  Then  FormulaXYS(d,x,y,s); { without approximation ArcSec }
   x:=SecRad*(x+DeltaXEmp); { with empirical correction X to radian }
   y:=SecRad*(y+DeltaYEmp); { with empirical correction Y to radian }
   x2:=Sqr(x); { x*x }
   y2:=Sqr(y); { y*y }
   cd:=Sqrt(1-(x2+y2)); { z=cos(d) , x y z is unit vector }
   aa:=1.0/(1+cd);
   q[1,1]:=+1-aa*x2;
   q[1,2]:=-aa*x*y;
   q[1,3]:=+x;
   q[2,1]:=-aa*x*y;
   q[2,2]:=+1-aa*y2;
   q[2,3]:=+y;
   q[3,1]:=-x;
   q[3,2]:=-y;
   q[3,3]:=+1-aa*(x2+y2); { this is part of Q(t) matrix }
   s:=SecRad*s; { a value for s(t) CEO position to radian }
   ForRotMatr(3,s,z); { to rotate about z-axis on the angle s(t) in radian }
   ToMultMatr(q,z,QMatr); { full Q(t) matrix unit UnForFun }
 End;

 { to multiplicate Q(t) and R(t) matrixes
   from the equator of the CIP to the fixed equator GCRS
   in this procedure binary file parmaps.pat
   with approximations coefficients is used }

 Procedure QRMapp2000 ( DateJulian   : Extended ;
                        DateJulUT1   : Extended ;
                        DeltaXEmp    : Extended ;
                        DeltaYEmp    : Extended ;
                        Var  QRMatr  : TMatr33  ) ;
 Var
   QMatr,RMatr  : TMatr33 ;
 Begin
   RMatr2000(DateJulUT1,RMatr); { unit Un2000_r }
   QMapp2000(DateJulian,DeltaXEmp,DeltaYEmp,QMatr);
   ToMultMatr(QMatr,RMatr,QRMatr); { full Q(t)*R(-theta) matrix }
 End;

 END.
