
unit UnRightS ; { right hand side of secular equations }

interface

uses
  UnForTyp ; { for type TVarSix }

procedure SecuRightHand ( tc  : Extended ;       { current moment in second }
                          ec  : TVarSix ;        { nonsingular elements }
                          var  fc  : TVarSix ) ; { forces for elements }

procedure SecuSolaPress ( tc  : Extended ;       { current moment in second }
                          ec  : TVarSix ;        { nonsingular elements }
                          var  fc  : TVarSix ) ; { forces for elements }

implementation

uses
  UnConTyp , { type TFundArg }
  UnTypVar , { type TValArg }
  UnVakElm , { for procedure SimpleElem function ToGetLonVelosity }
  UnitDeil , { deiler of TDeil for public ToGetDerSel d(a e \delta)(L G H) }
  UnVakNuF , { for procedure DerInertia }
  UnVakDes , { procedure ForDerParms ds/da ds/de ds/d\delta values }
  UnVakArg , { for procedure ToGetFundamArgu ToGetArgumValue }
  UnVakPow , { for procedure ToGetPowerValue to global arrays UnTypVar }
  UnVakFup , { for procedure ToGetForceValue ToGetForceSolap }
  UnForSop ; { for global variable satempac empiric acceleration }

{ to obtain numerical value
  for right hand side of secular equations
  all secular terms
  due to the geopotential
         the Moon the Sun with tides
      are in array rowcoh^[..] meccoh terms
  input
  tc current moment value in second of time
             from selected date in julian day
  output
  ec current values of nonsingular elements
  a , l+g+h , e*cos(g+h) , e*sin(g+h) , sin(i)*cos(h) , sin(i)*sin(h)
  fc values of the forces for this nonsingular elements
  global variables from UnForTyp
  DatTime of TDatTime with utc ut1 tt teph
  IntTime of TIntTime with begday cursec }

procedure SecuRightHand ( tc  : Extended ;       { current moment in second }
                          ec  : TVarSix ;        { nonsingular elements }
                          var  fc  : TVarSix ) ; { forces for elements }

var
  ii  : Byte ;     { count for variables }
  td  : Extended ; { current time in day }
  ra  : Extended ; { Earth rotation angle in radian }
  aa  : Extended ; { usual element a km }
  ee  : Extended ; { usual element e }
  si  : Extended ; { usual element sin(i) }
  ci  : Extended ; { usual element cos(i) }
  ul  : Extended ; { usual element angle l radian }
  ug  : Extended ; { usual element angle g radian }
  uh  : Extended ; { usual element angle h radian }
  ag  : TFundArg ; { UnConTyp fundamental argument [1..14] }
  ar  : TValArg ;  { UnTypVar value for arguments l,g,h,s_Z,\la,l,l',F,D }
begin
  for ii:=1 to NumberOfVar do  fc[ii]:=0; { initial nullo forces }
  { current moment in second relative to selected julian day moment }
  td:=IntTime.begday+(IntTime.cursec+tc)/86400.0; { to julian day }
  ToGetFundamArgu(td,ra,ag); { UnVakArg ERA and fundamental argum }
  { from nonsingular to usual elements with signum(cos(i)) }
  SimpleElem(ec,aa,ee,si,ci,ul,ug,uh); { UnVakElm with ceiler of TCeil class}
  { numerical value for arguments result to ar of TValArg }
  ToGetArgumValue(ul,ug,uh,ra,ag,ar); { from UnVakArg l,g,h,s_Z,\l,l,l',F,D }
  ToGetPowerValue(aa,ee,si,ci);       { from UnVakPow a^i e^i s^i c^i for i }
  deiler.ToGetDerSel ; { class TDeil UnitDeil for d(a e \delta)/d(L G H) }
  ForDerParms ; { global dersa derse dersd UnVakDes ds/da ds/de ds/d\delta }
  DerInertia(ar);  { UnVakNuF dfibg dF_i/dG dfibh dF_i/dH dfiah dF_i/dh }
  fc[2]:=ToGetLonVelosity ; { UnVakElm n_0*(1+\nu+\mu)/(1-\lambda) by ceiler }
  ToGetForceValue(ar,fc);   { from UnVakFup with global powers }
  { to add empiric acceleration to mean anomaly l value
    satempac empiric acceleration in revolution per day in square }
  fc[2]:=fc[2]+TwoPi*(2.0*satempac/Sqr(86400.0))*tc; { radian/sec }
end;

{ to obtain numerical value
  for right hand side of secular equations
  secular terms
  due to the Solar pressure
      are in array rowaux^[..] mecaux terms
  input
  tc current moment value in second of time
             from selected date in julian day
  output
  ec current values of nonsingular elements
  a , l+g+h , e*cos(g+h) , e*sin(g+h) , sin(i)*cos(h) , sin(i)*sin(h)
  fc values of the forces for this nonsingular elements
  global variables from UnForTyp
  DatTime of TDatTime with utc ut1 tt teph
  IntTime of TIntTime with begday cursec }


procedure SecuSolaPress ( tc  : Extended ;       { current moment in second }
                          ec  : TVarSix ;        { nonsingular elements }
                          var  fc  : TVarSix ) ; { forces for elements }
var
  td  : Extended ; { current time in day }
  ra  : Extended ; { Earth rotation angle in radian }
  aa  : Extended ; { usual element a km }
  ee  : Extended ; { usual element e }
  si  : Extended ; { usual element sin(i) }
  ci  : Extended ; { usual element cos(i) }
  ul  : Extended ; { usual element angle l radian }
  ug  : Extended ; { usual element angle g radian }
  uh  : Extended ; { usual element angle h radian }
  ag  : TFundArg ; { UnConTyp fundamental argument [1..14] }
  ar  : TValArg ;  { UnTypVar value for arguments l,g,h,s_Z,\la,l,l',F,D }
begin
  { current moment in second relative to selected julian day moment }
  td:=IntTime.begday+(IntTime.cursec+tc)/86400.0; { to julian day }
  ToGetFundamArgu(td,ra,ag); { UnVakArg ERA and fundamental argum }
  { from nonsingular to usual elements with signum(cos(i)) }
  SimpleElem(ec,aa,ee,si,ci,ul,ug,uh); { UnVakElm with ceiler of TCeil class}
  { numerical value for arguments result to ar of TValArg }
  ToGetArgumValue(ul,ug,uh,ra,ag,ar); { from UnVakArg l,g,h,s_Z,\l,l,l',F,D }
  ToGetPowerValue(aa,ee,si,ci);       { from UnVakPow a^i e^i s^i c^i for i }
  deiler.ToGetDerSel ; { class TDeil UnitDeil for d(a e \delta)/d(L G H) }
  ForDerParms ; { global dersa derse dersd UnVakDes ds/da ds/de ds/d\delta }
  ToGetForceSolap(ar,fc);   { from UnVakFup with global powers }
end;

end.
