
 { to make deal with the procedures
   to transform time-space systems of reference
   external variables from UnForTyp :
   IndTime , DatTime , IntTime ,
   DatPole , TerSelM , SelTerM , TerEquT , TruEquT }

 UNIT UnRotatM ;

 INTERFACE

 Procedure ToClcTimeValue ( td  : Extended ) ;

 Procedure ToObtainRotMats ( tc  : Extended ) ;

 Procedure ToObtainRotMatr ( tc  : Extended ) ;

 Procedure ToObtainRotMapp ( tc  : Extended ) ;

 Procedure ToObtainRotMapr ( tb , tc  : Extended ) ;

 Procedure ToObtaIntEquteM ( tc  : Extended ) ;

 IMPLEMENTATION

 Uses
   UnForTyp , { IndTime , DatTime , IntTime , DatPole , TerSelM }
   UnConTyp , { MatrUnit }
   UnForTim , { ToGetUT1 for example }
   UnGetERP , { to obtain empirical Earth rotation parameters }
   Un2000tc ; { matrixes with the IAU2000 resolution }

 { td is moment in julian day
   DatTime of TDatTime , DatPole of TDatPole  are declared in UnForTyp }

 Procedure ToClcTimeValue ( td  : Extended ) ;
 Var
   du  : Extended ; { deltaUT1=UT1-UTC }
   da  : Extended ; { difference TT-UTC }
 Begin
   If  IndTime = 1
     Then { our time scale is UTC in julian day }
       Begin { DatTime.(utc,ut1,tt,teph) in julian day }
         DatTime.utc:=td;
         DatPole.utc:=td;
         du:=DatPole.dut/86400.0; { DatPole.(utc,dut,xpo,...) }
         DatTime.ut1:=ToGetUT1(DatTime.utc,du); { UnForTim }
         FromUTCtoTT(DatTime.utc,DatTime.tt,da); { tt in TT }
         DatTime.teph:=ToGetTEph(DatTime.tt); { Teph }
       End;
 End;

 { tc is moment in second
   from the initial date that is in IntTime.begday
   DatTime.(utc,ut1,tt,teph)
   DatPole.(utc,dut,xpo,ypo,dxe,xye)
   result is in matrixes TerSelM(3,3) SelTerM(3,3) }

 Procedure ToObtainRotMats ( tc  : Extended ) ;
 Var
   td  : Extended ; { current data in julian day with the part }
 Begin
   td:=IntTime.begday+tc/86400.0; { current time in day }
   ToObtainERParm(td,DatPole); { ToGetERP , DatPole in UnForTyp }
   ToClcTimeValue(td); { result to DatTime , DatPole external }
   TerraSelestMats(DatTime.tt,DatTime.ut1,
                   DatPole.xpo,DatPole.ypo,
                   DatPole.dxe,DatPole.dye,
                   TerSelM,SelTerM); { procedure from Un2000tc }
 End;

 { more complex then previous procedure
   in TerraSelestMatr
   are obtained theoretical value for x_p y_p
   tc is moment in second
   from the initial date that is in IntTime.begday
   DatTime.(utc,ut1,tt,teph)
   DatPole.(utc,dut,xpo,ypo,dxe,xye)
   result is in matrixes TerSelM(3,3) SelTerM(3,3) }

 Procedure ToObtainRotMatr ( tc  : Extended ) ;
 Var
   td  : Extended ; { current data in julian day with the part }
 Begin
   td:=IntTime.begday+tc/86400.0; { current time in day }
   ToObtainERParm(td,DatPole); { ToGetERP , DatPole in UnForTyp }
   ToClcTimeValue(td); { result to DatTime , DatPole external }
   TerraSelestMatr(DatTime.tt,DatTime.ut1,
                   DatPole.xpo,DatPole.ypo,
                   DatPole.dxe,DatPole.dye,
                   TerSelM,SelTerM); { procedure from Un2000tc }
 End;

 { more complex then previous procedure
   in TerraSelestMapp
   Chebyshev approximation coefficients are used
   theoretical value for x_p y_p are obtained
   tc is moment in second
   from the initial date that is in IntTime.begday
   DatTime.(utc,ut1,tt,teph)
   DatPole.(utc,dut,xpo,ypo,dxe,xye)
   result is in matrixes TerSelM(3,3) SelTerM(3,3) }

 Procedure ToObtainRotMapp ( tc  : Extended ) ;
 Var
   td  : Extended ; { current data in julian day with the part }
 Begin
   td:=IntTime.begday+tc/86400.0; { current time in day }
   ToObtainERParm(td,DatPole); { ToGetERP , DatPole in UnForTyp }
   ToClcTimeValue(td); { result to DatTime , DatPole external }
   TerraSelestMapp(DatTime.tt,DatTime.ut1,
                   DatPole.xpo,DatPole.ypo, { empirical value x_p y_p }
                   DatPole.dxe,DatPole.dye, { empirical value dX , dY }
                   TerSelM,SelTerM); { procedure from Un2000tc }
 End;

 { more complex then previous procedure
   in TerraSelestMapp
   Chebyshev approximation coefficients are used
   theoretical value for x_p y_p are obtained
   tb is date in julian day
   tc is moment in second
   from the date that is in variable tb
   DatTime.(utc,ut1,tt,teph)
   DatPole.(utc,dut,xpo,ypo,dxe,xye)
   result is in matrixes TerSelM(3,3) SelTerM(3,3) UnForTyp }

 Procedure ToObtainRotMapr ( tb , tc  : Extended ) ;
 Var
   td  : Extended ; { current data in julian day with the part }
 Begin
   td:=tb+tc/86400.0; { current time in julian day }
   ToObtainERParm(td,DatPole); { ToGetERP , DatPole in UnForTyp }
   ToClcTimeValue(td); { result to DatTime , DatPole external }
   TerraSelestMapp(DatTime.tt,DatTime.ut1,
                   DatPole.xpo,DatPole.ypo, { empirical value x_p y_p }
                   DatPole.dxe,DatPole.dye, { empirical value dX , dY }
                   TerSelM,SelTerM); { procedure from Un2000tc }
 End;

 { to change from terrestrial system
      to the intermediate equator system }

 Procedure ToObtaIntEquteM ( tc  : Extended ) ;
 Var
   td  : Extended ; { current data in julian day with the part }
 Begin
   td:=IntTime.begday+tc/86400.0; { current time in day }
   ToObtainERParm(td,DatPole); { ToGetERP , DatPole in UnForTyp }
   ToClcTimeValue(td);   { result to DatTime , DatPole external }
   TerraIntEquMatr(DatTime.tt,DatTime.ut1,
                   DatPole.xpo,DatPole.ypo, { empirical value x_p y_p }
                   TerEquT,TruEquT); { procedure from Un2000tc }
 end;

 END.
