
unit UnStarto ; { to start process for the satellite }

interface

{ information from text file rowmuvao.inz
  output name of file with quick look data
  output start moment IntTime.begday .cursec
  output start nonsingular elements ElmStart
  output observations in binary file binqlos.tmp }

procedure ToStartProcess ;

{ information from text file rowmuvao.inz
  output name of file with quick look data
  output start moment IntTime.begday .cursec
  output start nonsingular elements ElmStart
  output observations in binary file binqlos.tmp }

procedure ToStartPromean ; { by using very middle elements }

procedure ToStartPrognoz ; { by using very middle elements }

implementation

uses
  UnConTyp , { type TVect3 }
  UnitCeil , { class TCeil with methods ceiler global }
  UnForEil , { Eiler elements Eiler intermediate orbit }
  UnHarMod , { for GemParmValue[1..4] }
  UnForDat , { TransDatetoJD }
  UnForSop , { global var satreflcoef function togetreflvalue }
  UnForQLo , { proc ToReadQuickLookFile }
  UnTideSt , { correction to site due to tides }
  UnForTyp , { for external IntTime record Pos Vel Start array }
  UnVakElm ; { proc ElemFromPosVel for six nonsingular elements }

procedure ToStartProcess ;
var
  fr  : TextFile ; { text file to read initial information }
  i  : Byte ; { count }
  nd,ny  : Integer ; { date day from start year and year }
  partday,dayearnew  : Extended ; { part in day day 01.01.year }
  datecur,datestart  : Extended ; { date for epoch date for start }
  ap,au,ai,ae,am,an  : Extended ; { elements }
  crefl,satmass,smidel  : Extended ; { satellite's parameters }
  ceil  : TDim90 ; { type from unit UnForEil }
  pos,vel  : TVect3 ; { type from UnCOnTyp }
  an0,a1,a2,a3  : Extended ; { for the procedures from UnForEil }
  edl,edg,edh,ul,ug,uh  : Extended ; { Eiler elements }
  difsec,ulc,ugc,uhc  : extended ; { angle l g h value start }
  stqloo  : shortstring ; { name of file with quick look data }
begin
  AssignFile(fr,'rowmuvao.inz'); // text file with information
  ReSet(fr); { to open to read elements from two-line elements format }
  ReadLn(fr,stoursat); { name of the satellite global UnForTyp}
  ReadLn(fr,ny);  { year }
  ReadLn(fr,nd);  { day from the begining of the year }
  ReadLn(fr,partday); { part in day }
  ReadLn(fr,ai);  { inclination in degree }
  ReadLn(fr,au);  { the ascending node in degree }
  ReadLn(fr,ae);  { eccentricity }
  ReadLn(fr,ap);  { argument of pericentre in degree }
  ReadLn(fr,am);  { mean anomaly in degree }
  ReadLn(fr,an);  { mean motion in revolution per day }
  ReadLn(fr,crefl);   { reflection coefficients }
  ReadLn(fr,satmass); { satellite's mass in kg }
  ReadLn(fr,smidel);  { satellite's midel area in m^2 }
  readln(fr,stqloo);  { name of file with quick look data }
  CloseFile(fr);  // that is all for our purpose
  TransDateToJD(1,1,ny,dayearnew); { 1 january for our year UnForDat }
  datecur:=dayearnew-1+nd+partday; { julian day epoch of the elements}
  satarama:=ToGetSataramas(crefl,smidel,satmass); { UnForSop in m^2/kg }
  satempac:=ToGetEmpaValue ; { UnForSop for empiric acceleration c_t }
  for i:=1 to 4 do
    ceil[i]:=GemParmValue[i]; { array GemParmValue[1..4] UnHarMod }
  an0:=2*Pi*an/86400; { mean motion from rev. per day to radian per second }
  ul:=Pi/180*am; { mean anomaly in radian }
  ug:=Pi/180*(ap+90); { argument of pericentre in radian }
  uh:=Pi/180*(au-90); { the ascending node in radian }
  FromElemToC(an0,ae,ai,ceil,a1,a2,a3); { unit UnForEil ai in degree }
  EilConst(ceil,a1,a2,a3,edl,edg,edh); { UnForEil orbit parameters }
  { to read to order data in quick look format }
  ToReadQuickLookFile(stqloo,datestart); // from UnForQLo observations
  { to transform data to binary file with corrections to sites positions }
  StatideCorr(datestart); // from UnTideSt corrections due to tides
  { datestart in julian day is start moment for improving process }
  difsec:=86400.0*(datestart-datecur); { in second for start date }
  ulc:=ul+ceil[24]*difsec; { angle l value radian for start date }
  ugc:=ug+ceil[25]*difsec; { angle g value radian for start date }
  uhc:=uh+ceil[26]*difsec; { angle h value radian for start date }
  orbxv(ceil,ulc,ugc,uhc,pos,vel); { position velosity in km km/s UnForEil }
  { global variable IntTime.begday UnForTyp start moment
    time is account in second from this value as nullo point }
  IntTime.begday:=datestart; { initial moment for elements in julian day }
  IntTime.cursec:=0.0; { second from initial moment  IntTime  UnForTyp }
  DayStart:=datestart; { global variable UnForTyp epoch julian day }
  PosStart:=pos; { initial position global UnForTyp at start moment }
  VelStart:=vel; { initial velosity global UnForTyp at start moment }
  CofStart:=satarama; { global UnForTyp reflection coefficient k_r }
  { procedure from UnVakElm
    global input  Pos Vel Start
           output ElmStart UnForTyp
    from position velosity to nonsingular elements }
  ElemFromPosVel(PosStart,VelStart,ElmStart); { six nonsingular elements }
end;

{ information from text file rowmuvao.inz
  output name of file with quick look data
  output start moment IntTime.begday .cursec
  output start nonsingular elements ElmStart
  output observations in binary file binqlos.tmp }

procedure ToStartPromean ; { by using very middle elements }
var
  fr     : TextFile ; { text file to read initial information }
  nd,ny  : Integer ; { date day from start year and year }
  partday,dayearnew  : Extended ; { part in day day 01.01.year }
  datecur,datestart  : Extended ; { date for epoch date for start }
  ap,au,ai,ae,am,an  : Extended ; { elements }
  crefl,cempa        : Extended ; { satellite's parameters }
  difsec,ul,ug,uh    : Extended ; { Eiler angle elements }
  ulc,ugc,uhc,sumu   : extended ; { angle l g h value start }
  stqloo             : shortstring ; { name of file with quick look data }
begin
  AssignFile(fr,'rowmuvao.inz'); // text file with information
  ReSet(fr); { to open to read elements from two-line elements format }
  ReadLn(fr,stoursat); { name of the satellite global UnForTyp}
  ReadLn(fr,ny);  { year }
  ReadLn(fr,nd);  { day from the begining of the year }
  ReadLn(fr,partday); { part in day }
  ReadLn(fr,ai);  { inclination in degree }
  ReadLn(fr,au);  { the ascending node in degree }
  ReadLn(fr,ae);  { eccentricity }
  ReadLn(fr,ap);  { argument of pericentre in degree }
  ReadLn(fr,am);  { mean anomaly in degree }
  ReadLn(fr,an);  { mean motion in revolution per day }
  ReadLn(fr,crefl);   { reflection coefficients k_r*A_m/m_s in m^2/kg }
  ReadLn(fr,cempa);   { empiric acceleration revolution per day per day }
  readln(fr,stqloo);  { name of file with quick look data }
  CloseFile(fr);  // that is all for our purpose
  TransDateToJD(1,1,ny,dayearnew); { 1 january for our year UnForDat }
  datecur:=dayearnew-1+nd+partday; { julian day epoch of the elements}
  satarama:=crefl; { global UnForSop for reflection coefficient in m^2/kg }
  satempac:=cempa; { global UnForSop for empiric acceleration in rev/day^2 }
  ul:=Pi/180*am; { mean anomaly in radian }
  ug:=Pi/180*(ap+90); { argument of pericentre in radian }
  uh:=Pi/180*(au-90); { the ascending node in radian }
  ceiler.FromElemToCeil(an,ae,ai); { UnitCeil method of class TCeil }
  { to read to order data in quick look format }
  ToReadQuickLookFile(stqloo,datestart); // from UnForQLo observations
  { to transform data to binary file with corrections to sites positions }
  StatideCorr(datestart); // from UnTideSt corrections due to tides
  { datestart in julian day is start moment for improving process }
  difsec:=86400.0*(datestart-datecur); { in second for start date }
  ulc:=ul+ceiler.c[24]*difsec; { angle l value radian for start date }
  ugc:=ug+ceiler.c[25]*difsec; { angle g value radian for start date }
  uhc:=uh+ceiler.c[26]*difsec; { angle h value radian for start date }
  sumu:=ulc+ugc+uhc; { l+g+h sum of three angles in radian UnConTyp }
  while  sumu <  0.0   do  sumu:=sumu+TwoPi; { sum of three angles }
  while  sumu > TwoPi  do  sumu:=sumu-TwoPi; { to usual interval }
  { global variable IntTime.begday UnForTyp start moment
    time is account in second from this value as nullo point }
  IntTime.begday:=datestart; { initial moment for elements in julian day }
  IntTime.cursec:=0.0; { second from initial moment  IntTime  UnForTyp }
  DayStart:=datestart; { global variable UnForTyp epoch julian day }
  CofStart:=satarama; { global UnForTyp reflection coefficient k_r }
  { to get six very middle nonsingular elements }
  ElmStart[1]:=ceiler.c[8]; { a  semi-major axis in km }
  ElmStart[2]:=sumu;        { l+g+h sum of three angles in radian }
  ElmStart[3]:=ceiler.c[9]*cos(ugc+uhc); { e*cos(g+h) }
  ElmStart[4]:=ceiler.c[9]*sin(ugc+uhc); { e*sin(g+h) }
  ElmStart[5]:=ceiler.c[38]*cos(uhc); { \delta*cos(h) }
  ElmStart[6]:=ceiler.c[38]*sin(uhc); { \delta*sin(h) }
end;

{ information from text file rowmuvap.inz
  output name of file with quick look data
  output start moment IntTime.begday .cursec
  output start nonsingular elements ElmStart
  output observations in binary file binqlos.tmp }

procedure ToStartPrognoz ; { by using very middle elements }
var
  fr     : TextFile ; { text file to read initial information }
  nd,ny  : Integer ; { date day from start year and year }
  partday,dayearnew  : Extended ; { part in day day 01.01.year }
  datecur,datestart  : Extended ; { date for epoch date for start }
  ap,au,ai,ae,am,an  : Extended ; { elements }
  crefl,cempa        : Extended ; { satellite's parameters }
  difsec,ul,ug,uh    : Extended ; { Eiler angle elements }
  ulc,ugc,uhc,sumu   : extended ; { angle l g h value start }
  stqloo             : shortstring ; { name of file with quick look data }
begin
  AssignFile(fr,'rowmuvap.inz'); // text file with information
  ReSet(fr); { to open to read elements from two-line elements format }
  ReadLn(fr,stoursat); { name of the satellite global UnForTyp}
  ReadLn(fr,ny);  { year }
  ReadLn(fr,nd);  { day from the begining of the year }
  ReadLn(fr,partday); { part in day }
  ReadLn(fr,ai);  { inclination in degree }
  ReadLn(fr,au);  { the ascending node in degree }
  ReadLn(fr,ae);  { eccentricity }
  ReadLn(fr,ap);  { argument of pericentre in degree }
  ReadLn(fr,am);  { mean anomaly in degree }
  ReadLn(fr,an);  { mean motion in revolution per day }
  ReadLn(fr,crefl);   { reflection coefficients k_r*A_m/m_s in m^2/kg }
  ReadLn(fr,cempa);   { empiric acceleration revolution per day per day }
  readln(fr,stqloo);  { name of file with quick look data }
  CloseFile(fr);  // that is all for our purpose
  TransDateToJD(1,1,ny,dayearnew); { 1 january for our year UnForDat }
  datecur:=dayearnew-1+nd+partday; { julian day epoch of the elements}
  satarama:=crefl; { global UnForSop for reflection coefficient in m^2/kg }
  satempac:=cempa; { global UnForSop for empiric acceleration in rev/day^2 }
  ul:=Pi/180*am; { mean anomaly in radian }
  ug:=Pi/180*(ap+90); { argument of pericentre in radian }
  uh:=Pi/180*(au-90); { the ascending node in radian }
  ceiler.FromElemToCeil(an,ae,ai); { UnitCeil method of class TCeil }
  { to read to order data in quick look format }
  ToReadQuickLookFile(stqloo,datestart); // from UnForQLo observations
  { to transform data to binary file with corrections to sites positions }
  StatideCorr(datestart); // from UnTideSt corrections due to tides
  { datestart in julian day is start moment for improving process }
  difsec:=86400.0*(datestart-datecur); { in second for start date }
  ulc:=ul+ceiler.c[24]*difsec; { angle l value radian for start date }
  ugc:=ug+ceiler.c[25]*difsec; { angle g value radian for start date }
  uhc:=uh+ceiler.c[26]*difsec; { angle h value radian for start date }
  sumu:=ulc+ugc+uhc; { l+g+h sum of three angles in radian UnConTyp }
  while  sumu <  0.0   do  sumu:=sumu+TwoPi; { sum of three angles }
  while  sumu > TwoPi  do  sumu:=sumu-TwoPi; { to usual interval }
  { global variable IntTime.begday UnForTyp start moment
    time is account in second from this value as nullo point }
  IntTime.begday:=datestart; { initial moment for elements in julian day }
  IntTime.cursec:=0.0; { second from initial moment  IntTime  UnForTyp }
  DayStart:=datestart; { global variable UnForTyp epoch julian day }
  CofStart:=satarama; { global UnForTyp reflection coefficient }
  { to get six very middle nonsingular elements }
  ElmStart[1]:=ceiler.c[8]; { a  semi-major axis in km }
  ElmStart[2]:=sumu;        { l+g+h sum of three angles in radian }
  ElmStart[3]:=ceiler.c[9]*cos(ugc+uhc); { e*cos(g+h) }
  ElmStart[4]:=ceiler.c[9]*sin(ugc+uhc); { e*sin(g+h) }
  ElmStart[5]:=ceiler.c[38]*cos(uhc); { \delta*cos(h) }
  ElmStart[6]:=ceiler.c[38]*sin(uhc); { \delta*sin(h) }
end;

end.
