
unit UnTideSt ; // to read binqlo.tmp file

interface       // to obtain station correction due tides

uses
  UnConTyp ;    // type TVect3 const GeoFM
                 // information about stations of observations
const           
  msite  = 169 ; // maximum number of station
var              // global variables about stations
  ksite  : Word ;                         // number of stations
  nsite  : array [1..msite] of SmallInt ; // stations identifications
  psite  : array [1..msite] of tvect3 ;   // stations positions
  siter  : array [1..msite] of extended ; // geocentric range km
  sitef  : array [1..msite] of extended ; // geodetic latitude phi radian
  sitel  : array [1..msite] of extended ; // geodetic longitude lambda
  siteh  : array [1..msite] of extended ; // geodetic height km

procedure AboutStations ; // to read identifications and coordinates

procedure StatideCorr ( start  : extended ) ; // for station position

implementation

uses
  UnForTyq , // types TRecObr global variable recobr
  UnForCoo , // procedure ClcSpherCoor
  UnForFun , // procedure MultMatrVec
  UnEph405 , // for the Moon the Sun position
  UnRotatM , // complex procedure ToObtainRotMapr
  UnForTyp , // global variables SelTerM
  UnForPrt ; // for protocol text file

var
  fr  : file of TRecObs ; // UnForTyq for recobs global file to read
  fw  : file of TRecObr ; // UnForTyq for recobr global file to write

procedure AboutStations ; // to read identifications and coordinates
var
  f  : TextFile ; // text file stndoi.pgm for stations
  n  : SmallInt ; // current number of site
  x  : extended ; // station coordinate x in metr
  y  : extended ; // station coordinate y in metr
  z  : extended ; // station coordinate z in metr
begin
  AssignFile(f,'stndoi.pgm'); // for stations
  ReSet(f);  // to read
  ReadLn(f); // without the first string
  ksite:=0;  // initial nullo count
repeat       // station by station
  ReadLn(f,n,x,y,z); // station number position vector metr
  ksite:=ksite+1;  // for the next station
  if  ksite > msite  then  ksite:=msite; // without strange case
  nsite[ksite]:=n; // station number
  psite[ksite][1]:=1.0e-3*x; // site coordinate x in km
  psite[ksite][2]:=1.0e-3*y; // site coordinate y in km
  psite[ksite][3]:=1.0e-3*z; // site coordinate z in km
  siter[ksite]:=1.0e-3*Sqrt(Sqr(x)+Sqr(y)+Sqr(z)); // geocentric range km
  GeodeticSpher(x,y,z,         // UnForCoo input coordinates in metr
                sitef[ksite],  // output geodetic latitude radian
                sitel[ksite],  // output geodetic longitude radian
                siteh[ksite]); // output geodetic height kilometer UnForCoo
until  EOF(f); // the end of our text file with coordinates
  CloseFile(f); // after reading
end; // AboutStations  number of all stations equal ksite

{ any celestial body position to terrestrial system }
procedure BodyToTerra ( np  : Byte ; // body identificator
                        var  ar , af , al  : extended ; // spheric
                        var  ue  : tvect3 ) ; // unit vector terra
var
  i   : byte ;     // simple count
  tp  : extended ; // moment in T_eph from DatTime
  pe  : tvect3 ;   // the Earth barycentric position
  vv  : tvect3 ;   // temporary velosity
  pp  : tvect3 ;   // our np body position
  pb  : tvect3 ;   // body terrestrial position
begin
  tp:=DatTime.teph; // global DatTime from UnForTyp
  if  np = NMoon    // const UnEph405
    then            // the Moon celestial  UnEph405
      PosVel405(np,tp,pp,vv) // refer to the Earth 
    else                     // but other bodies
      begin  // refer to the barycenter celestial system
        EarthB405(tp,pe,vv); // UnEph405 the Earth celestial barycentric
        PosVel405(np,tp,pp,vv); // UnEph405 body refer to the barycenter 
        for i:=1 to 3 do     // for three coordinates
          pp[i]:=pp[i]-pe[i]; // to the center of the Earth 
      end;
  MultMatrVec(SelTerM,pp,pb); // UnForFun UnForTyp to terra
  ClcSpherCoor(pb,al,af,ar); // spheric coordinates radian km
  for i:=1 to 3 do   // for three coordinates
    ue[i]:=pb[i]/ar; // unit body vector terrestrial
end;

{ to obtain the main correction due tides
     with Love number h_2(phi) l_2(phi) result in recobr.ot[..] }
procedure TideDegreeTwo ( np  : byte ;     // body identificator
                          rp  : extended ; // body geocentric range
                          up  : tvect3 ;   // body terra unit vector
                          rs  : extended ; // site geocentric range
                          sf  : extended ; // site sinus latitude
                          ur  : tvect3 ) ; // site terra unit vector
var
  i  : byte ; // count for three coordinates
  qf,qr  : extended ; // temporary Love or Shida numbers
  vh,vl  : extended ; // value for Love numbers h_2(phi) l_2(phi)
begin
  qf:=1.5*Sqr(sf)-0.5;   // temporary   P_2(sin(phi))
  vh:=0.6078-0.0006*qf;  // Love number h_2 as function latitude
  vl:=0.0847+0.0002*qf;  // Love number l_2 as function latitude
  qr:=ScalarMult(up,ur); // function UnForFun two unit vectors
  { factor in formulae (fm_j/fm)*(r^4/r_j^3) UnEph405 UnConTyp }
  qf:=FMPlanet[np]/FMPlanet[NEarth]; // ratio for masses
  qf:=qf*Sqr(rs)*Sqr(rs)/(rp*Sqr(rp)); // fm_j/fm...
  vh:=qf*vh*(1.5*Sqr(qr)-0.5); // value with factor
  vl:=3.0*qf*vl*qr;            // value with factor
  for i:=1 to 3 do  // for three coordinates
    recobr.ot[i]:=recobr.ot[i]+vh*ur[i]+vl*(up[i]-qr*ur[i]);
end; // TideDegreeTwo procedure

{ to obtain the correction due three order tides
     with Love number h_3 l_3 result in recobr.ot[..] }
procedure TideDegreeTre ( np  : byte ;     // body identificator
                          rp  : extended ; // body geocentric range
                          up  : tvect3 ;   // body terra unit vector
                          rs  : extended ; // site geocentric range
                          ur  : tvect3 ) ; // site terra unit vector
var
  i  : byte ; // count for three coordinates
  qf,qr  : extended ; // temporary three order
  vh,vl  : extended ; // value for Shida numbers h_3 l_3
begin
  vh:=0.292;  // Love Shida number h_3
  vl:=0.015;  // Love Shida number l_3
  qr:=ScalarMult(up,ur); // function UnForFun two unit vectors
  { factor in formulae (fm_j/fm)*(r^5/r_j^4) UnEph405 UnConTyp }
  qf:=FMPlanet[np]/FMPlanet[NEarth]; // ratio for masses
  qf:=qf*rs*Sqr(rs)*Sqr(rs)/(Sqr(rp)*Sqr(rp)); // factor
  vh:=qf*vh*qr*(2.5*Sqr(qr)-1.5); // value with factor
  vl:=qf*vl*(7.5*Sqr(qr)-1.5);    // value with factor
  for i:=1 to 3 do  // for three coordinates
    recobr.ot[i]:=recobr.ot[i]+vh*ur[i]+vl*(up[i]-qr*ur[i]);
end; // TideDegreeTre procedure

{ to obtain correction to station position
  and to fill new record with those information }
procedure ForTideCorr( r , a , b  : extended ) ; // r phi lambda radian
var
  sf,cf  : extended ; // sinus cosinus of latitude
  sl,cl  : extended ; // sinus cosinus of longitude
  ur,ue,un  : tvect3 ; // unit vectors in three directions
  um,uz  : tvect3 ;   // geocentric unit vectors for the Moon the Sun
  rm,am,bm  : extended ; // geocentric spheric position of the Moon
  rz,az,bz  : extended ; // geocentric spheric position of the Sun
begin
  cf:=Cos(a); // cosinus of geodetic latitude  argument in radian
  sf:=Sin(a); //   sinus of geodetic latitude  argument in radian
  cl:=Cos(b); // cosinus of geodetic longitude argument in radian
  sl:=Sin(b); //   sinus of geodetic longitude argument in radian
  ur[1]:=+cf*cl; // geocentric unit vector of site coordinate x
  ur[2]:=+cf*sl; // geocentric unit vector of site coordinate y
  ur[3]:=+sf;    // geocentric unit vector of site coordinate z
  ue[1]:=-sl;    // geocentric unit vector of site direct to east x
  ue[2]:=+cl;    // geocentric unit vector of site direct to east y
  ue[3]:=0.0;    // geocentric unit vector of site direct to east z
  un[1]:=-sf*cl; // geocentric unit vector of site direct to nord x
  un[2]:=-sf*sl; // geocentric unit vector of site direct to nord y
  un[3]:=+cf;    // geocentric unit vector of site direct to nord z
  BodyToTerra(NMoon,rm,am,bm,um); // const UnEph405 with TerSelM
  BodyToTerra(NSun,rz,az,bz,uz);  // const UnEph405 and DatTime
  TideDegreeTwo(NMoon,rm,um,r,sf,ur); // with Love number h_2 l_2(phi)
  TideDegreeTwo(NSun,rz,uz,r,sf,ur);  // with Love number h_2 l_2(phi)
  TideDegreeTre(NMoon,rm,um,r,ur); // with Shida number h_3 l_3
end;

{ to find our station position in the list
  and to fill new record with those information }
function BooSiteCoor ( n  : SmallInt ) : Boolean ;
var
  i  : Word ;    // count to select station
  b  : Boolean ; // temporary
begin
  b:=False; // for a try to find station in list
  i:=0;     // initial to find
repeat      // a try to find station number in list
  i:=i+1;   // the next station from our list
  if  ( n = nsite[i] )  then  // to compare two numbers
    begin                     // input and from our list
      b:=True;                // this is our number
      recobr.oz:=psite[i];    // coordinates of our station TVect3
      recobr.zp:=sitef[i];    // geodetic latitude  radian
      recobr.zl:=sitel[i];    // geodetic longitude radian
      recobr.zh:=siteh[i];    // geodetic height kilometer
      ToObtainRotMapr(recobs.dm,recobs.sp); // UnRotatM to SelTerM
      { global variable DatPole for ERP UnForTyp UnGetErp }
      recobr.zu:=DatPole.dut/86400.0; // delta(UT) in day
      recobr.zx:=SecRad*DatPole.xpo;  // x_pole in radian
      recobr.zy:=SecRad*DatPole.ypo;  // y_pole in radian
      recobr.ot[1]:=0.0;  // intitial nullo tide correction x
      recobr.ot[2]:=0.0;  // intitial nullo tide correction y
      recobr.ot[3]:=0.0;  // intitial nullo tide correction z
      ForTideCorr(siter[i],sitef[i],sitel[i]); // r phi lambda
    end;                      // this is our number
until  ( b or ( i = ksite ) ) ; // may be our station
  BooSiteCoor:=b;
end; // ToStideCorr procedure

{ to transform some information
  from one record to more complex record }
procedure DataFromRecObs ( start  : extended ) ;
begin        { global variable record UnForTyq }
  recobr.ns:=recobs.ns; { number of the satellite   LongInt }
  recobr.no:=recobs.no; { number of the observatory SmallInt }
  recobr.dd:=start; { start moment in julian day sd second from dd }
  recobr.sd:=+86400.0*Round(recobs.dm-start)  { to second of time }
             +recobs.sp      { part from dm in second }
             +0.5*recobs.ds; { moment on satellite in second }
  recobr.ro:=0.5*VelOfLight*recobs.ds; { topocentric range in kilometer }
  recobr.wv:=1.0e-4*recobs.lw;  { length of way from A to mikrometer }
  recobr.pr:=0.1*recobs.lp; { athmospheric pressure to millibar }
  recobr.hp:=1.0*recobs.lh; { athmospheric humidity in percent }
  recobr.tk:=0.1*recobs.lt; { temperature in Kelvin scale }
end;

{ initial nullo value to recobr.zs[..]
  to middle nonsungular elements short periodic adding }
procedure ShortNulloCorr ; // nullo short periodic correction
begin { to six nonsingular elements }
  recobr.zs[1]:=0.0; { a }
  recobr.zs[2]:=0.0; { l+g+h }
  recobr.zs[3]:=0.0; { e*cos(g+h) }
  recobr.zs[4]:=0.0; { e*sin(g+h) }
  recobr.zs[5]:=0.0; { \delta*cos(h) }
  recobr.zs[6]:=0.0; { \delta*sin(h) }
end;

{ to obtain station position and to correct with tides }
procedure StatideCorr ( start  : extended ) ; // for station position
var                   { start moment to start process }
  kr,kw  : longint ; // simple counts
begin
  AboutStations ; // sites numbers and positions from stndoi.pgm
  AssignFile(fr,'binqlo.tmp'); // binary file with observations
  ReSet(fr); // to open to read
  AssignFile(fw,'binqlos.tmp'); // new binary file
  ReWrite(fw); // to open to write
  kr:=0; // initial nullo for records to read
  kw:=0; // initial nullo for records to write
repeat               // to read file with observations
  read(fr,recobs);   // the next record from quick look file
  kr:=kr+1;          // simple count for record to read
  DataFromRecObs(start);     // part of recobs to recobr record
  ShortNulloCorr ;           // nullo short periodic correction
  if  BooSiteCoor(recobs.no) // information to record recobr
    then                     // this number is in our list
      begin                  // but may be that no number
        write(fw,recobr);    // the next record with coordinates
        kw:=kw+1;            // record is prepare to write
      end
    else                     // no this number in our list of stations
      WriteLn(PrtFile,recobs.no:6,'  no this site number in list');
until  EOF(fr) ; // the end of observations file binqlo.tmp
  CloseFile(fw); // our new file with information
  CloseFile(fr); // that is all
  Erase(fr);     // to delete this usefull file
  WriteLn ;
  WriteLn(kr:10,kw:10,
          '  records to read to write with corrections');
  WriteLn(PrtFile);
  WriteLn(PrtFile,kr:10,kw:10,
                  '  records to read to write with corrections');
end; // procedure StatideCorr

end.
