{ from celestial to terrestrial system
  by using simple matrix SelTerM declared UnForTyp
  simple legend about variables
  unit UnMatDer procedure ToObtainDerivations
  for the value NumD=0
      is read current record recobr
      is called procedure ToObtainRotMapr (UnRotatM)
  it is procedure the matrix to change SelTerM is calculated }
unit UnTusRoC ; { to calculate topocentric range }
interface       { in terrestrial system }
uses
  UnForTyp ; { for type TVarSix global matrix to change TruEquT[3,3] }
{ input  tc moment in second from start julian day
         ec nonsingular elements  of the satellite }
function ToGetTopoRange ( tc  : extended ;  ec  : tvarsix ) : extended ;
implementation
uses
  UnConTyp , { type TVect3 }
  UnForTyq , { global record recobr with information about site }
  UnForFun , { simple function to multiply matrix and vector }
  UnVakOsc ; { for procedure PosVelOscu position from elements }
{ correction to calculated value
             of range due to refraction
  algorithm  presented by
             doctor Marini and doctor Murray
    by using meteodata from current record recobr }
function ToGetRefraCorr ( sh  : extended ) : extended ; { sin(h) }
var
  wo,wt,cf,fl,fh,tc,po,qe,qk,rk,aa,bb,ab,rr  : extended ; { temp var }
begin
  ToGetRefraCorr:=0.0; { initial default nullo correction }
  if  recobr.wv = 0.0  then  exit ; { nullo length of wave no action }
  if  recobr.pr = 0.0  then  exit ; { nullo pressure no action }
  if  recobr.hp = 0.0  then  exit ; { nullo humidity no action }
  if  recobr.tk = 0.0  then  exit ; { nullo temperature no action }
  cf:=Cos(2*recobr.zp); { cos(2*latitude }
  wo:=1.0/recobr.wv;  { 1/lambda }
  wt:=Sqr(wo);        { 1/lambda^2 }
  fl:=0.9650+(0.0164+0.000228*wt)*wt; { function lambda }
  fh:=1.0-0.0026*cf-0.00031*recobr.zh; { function height }
  tc:=recobr.tk-273.15;  { temperature Celsium }
  po:=7.5*tc/(237.3-tc); { power with temperature }
  qe:=0.0611*recobr.hp*Exp(po*Ln(10)); { with humidity and power }
  qk:=1.163-0.00968*cf-0.00104*recobr.tk+0.1435e-4*recobr.pr; { value K }
  rk:=2.0/(3.0-1.0/qk); { factor with latitude temperature pressure }
  aa:=0.002357*recobr.pr+0.000141*qe; { value A in formula }
  bb:=1.084e-8*recobr.pr*recobr.tk*qk      { in formula }
      +4.734e-8*Sqr(recobr.pr)*rk/recobr.tk; { value B }
  ab:=aa+bb; { simple A+B form formula }
  rr:=bb/(ab/(sh+0.01)); { B/((A+B)/(sin(h)+0.01)) }
  ToGetRefraCorr:=1.0e-3*(fl/fh)*ab/(sh+rr); { kilometer }
end;
{ range between
  the center of satellite and the center of reflection }
function ToGetDroValue : extended ; { in kilometer with recobr.ns value }
var
  r  : extended ;
begin
  r:=0.5; { default value in meter }
  if  recobr.ns = 7603901  then  r:=0.240; { in meter Lageos }
  if  recobr.ns = 9207002  then  r:=0.240; { in meter Lageos-2 }
  if  recobr.ns = 7501001  then  r:=0.075; { in meter Startlett }
  if  recobr.ns = 9306102  then  r:=0.075; { in meter Stella }
  if  recobr.ns = 8900103  then  r:=0.558; { in meter Etalon-1 }
  if  recobr.ns = 8903903  then  r:=0.558; { in meter Etalon-2 }
  ToGetDroValue:=1.0e-3*r; { range in kilometer }
end;
{ input  tc moment in second from start julian day
         ec nonsingular elements  of the satellite }
function ToGetTopoRange ( tc  : extended ;  ec  : tvarsix ) : extended ;
var
  ro  : extended ; { topocentric range in terrestrial system }
  rc  : tvect3 ;   { satellite position vector inter.equator system }
  vc  : tvect3 ;   { satellite velosity vector inter.equator system }
  rz  : tvect3 ;   { terrestrial geocentric vector of the satellite }
  pz  : tvect3 ;   { terrestrial geocentric site position with tide corr. }
  rp  : tvect3 ;   { terrestrial topocentric satellite position }
  sh  : extended ; { approximately value for sinus altitude above horizon }
  cf  : extended ; { for station cosinus latitude }
  sf  : extended ; { for station   sinus latitude }
  cl  : extended ; { for station cosinus longitude }
  sl  : extended ; { for station   sinus longitude }
begin { information about station in current record recobr }
  PosVelOscu(ec,rc,vc); { UnVakOsc }
  { satellite position from true equator to terrestrial system }
  MultMatrVec(TruEquT,rc,rz); { terra satellite position kilometer }
  { terra station position with correction due to tides }
  pz[1]:=recobr.oz[1]+recobr.ot[1]; { site coordinate x }
  pz[2]:=recobr.oz[2]+recobr.ot[2]; { site coordinate y }
  pz[3]:=recobr.oz[3]+recobr.ot[3]; { site coordinate z }
  { now satellite vector rz and site vector pz
    are in the same geocentric terrestrial system }
  rp[1]:=rz[1]-pz[1]; { satellite refer to station x }
  rp[2]:=rz[2]-pz[2]; { satellite refer to station y }
  rp[3]:=rz[3]-pz[3]; { satellite refer to station z }
  ro:=Sqrt(Sqr(rp[1])+Sqr(rp[2])+Sqr(rp[3])); { range from site }
  { about site from recobr }
  cf:=Cos(recobr.zp); { cosinus latitude }
  sf:=Sin(recobr.zp); {   sinus latitude }
  cl:=Cos(recobr.zl); { cosinus longitude }
  sl:=Sin(recobr.zl); {   sinus longitude }
  { approximately value for sinus altitude above horizon }
  sh:=(cf*cl*rp[1]+cf*sl*rp[2]+sf*rp[3])/ro; { simple scalar multiply }
  ToGetTopoRange:=ro+ToGetRefraCorr(sh)  { ro_C value with refraction }
                    -ToGetDroValue;      { without center range }
end;
end.
