
{ a satellite of the Earth
  for example case expansion with Bessel function }

unit UnTypVar ; { for different types for some case }

interface       { a satellite of the Earth }

uses
  Classes  ; { for TMemoryStream }

const
  maxarr = 54967 ;  // dimension of arrays
  maxpec = +10 ;    // maximum power for eccentricity
  minaxi = -20 ;    // minimum negative power a
  maxaxi =  +7 ;    // maximum positive power a
  maxpsc = +17 ;    // maximum positive power sin(i) and cos(i)

  numsel = 4 ;      // selected elements  nullo e  sin(i)  cos(i)  in use
  maxsel : array[1..numsel] of ShortInt = // maximum power
           ( 25 ,   // a                 in use
             10 ,   // e                 in use
             25 ,   // s=sin(i)          in use
             25 ) ; // c=cos(i)          in use

  maxarg = 9 ;  // number of arguments l g h S_Z \lambda l_M l'_S F D'
  maxder = 6 ;  // amplitude with derivations on a - to maxder derivations
  periom = 2.0 ; // in day to select short and long periodic terms
  fulstp = 0.25 ;  // step in day to integrate secular equations
  coefam = 1.0e-10 ; // to neglect term with amplitude and power 0.1

type
  TAmpDer  = array[1..maxder] of Extended ; { amplitude maxder elements }
  TDimSel  = array[1..numsel] of ShortInt ; { for power of var. 4 bytes }
  TDimArg  = array[1..maxarg] of ShortInt ; { for arguments 9 bytes }
  { l=M g=omega+Pi/2  h=Omega-Pi/2 in our case
    where
    Omega is the ascending node
    omega is argument f perigei
    M is mean anomaly as usual kepler angle eleme nts }
  TValArg  = array[1..maxarg] of Extended ; { value for maxarg angles }

  { elementary term has a form
        A    numerical amplitude and derivations with respect to a
         *e^i_2                      eccentricity max power is limited
         *(sin i)^i_3                sinus inclination without limit
         *(cos i)^i_4                cosinus inclination without limit
          * Cos or Sin               cosinus or sinus of the next argument
             (k_1*l+k_2*g+k_3*h      for the satellite angle
              +k_4*S_Z               sidereal time k_4=-k_3 or k_4=0
              +k_5*\lambda+k_6*l_M   for the Moon the Sun
              +k_7*l'_S+k_8*F+k_9*D) fundamental arguments }

  TRowVar  = record
               amp  : TAmpDer ; { amplitude derivat. 10*maxder bytes }
               mec  : TDimSel ; { power of variables   4 bytes }
               mar  : TDimArg ; { coef. for argument   9 bytes }
               fcs  : Char ;    { function sin or cos  1 byte  }
             end;

type
  { for the array }
  TRowDim  = array[1..maxarr] of TRowVar ;

type
  { for the pointer to array }
  TRowPtr  = ^TRowDim ;

var
  memrec  : Word ;    { size of record TRowVar }

var
  maxsum  : ShortInt ; { maximum for sum of powers }
  maxpow  : ShortInt ; { current maximum power of eccentricity }
  recnul  : TRowVar ; { for nullo record type from UnTypVar }
  mecone  : Word ;    { number of terms in rowone array }
  mectwo  : Word ;    { number of terms in rowtwo array }
  mectre  : Word ;    { number of terms in rowtre array }
  mecfur  : Word ;    { number of terms in rowfur array }
  mecfiv  : Word ;    { number of terms in rowfiv array }
  mecsix  : Word ;    { number of terms in rowsix array }
  mecaux  : Word ;    { number of terms in rowaux array }
  mectmp  : Word ;    { number of terms in tempor array }
  meccoh  : Word ;    { number of terms in rowcoh array }
  rowone  : TRowPtr ; { to the pointer for array }
  rowtwo  : TRowPtr ; { to the pointer for array }
  rowtre  : TRowPtr ; { to the pointer for array }
  rowfur  : TRowPtr ; { to the pointer for array }
  rowfiv  : TRowPtr ; { to the pointer for array }
  rowsix  : TRowPtr ; { to the pointer for array }
  rowaux  : TRowPtr ; { to the pointer for array }
  rowtmp  : TRowPtr ; { pointer for tempor array }
  rowcoh  : TRowPtr ; { pointer for rowcoh array }
var
  signuc  : ShortInt ; { global variable Signum(cos(i)) +1 or -1 }
  axinul  : Extended ; { global variable the mean semimajor axis value }
  axicur  : Extended ; { global variable for current a value }
  eccmow  : array[0..maxpec] of Extended ; { numerical value e^n }
  aximow  : array[minaxi..maxaxi] of Extended ; { num. value a^n }
  sinmow  : array[0..maxpsc] of Extended ; { numerical value s^n s=sin(i) }
  cosmow  : array[0..maxpsc] of Extended ; { numerical value c^n c+cos(i) }
  tremot  : array[1..3] of Extended ; { d(l g h)/dt in radian per second }

var
  perioz  : extended ; // period in day to select periodic terms
  solves  : Extended ; // in second step to integrate evolutional equations
  { value of eccentricity in some powers 0..10 }
  epower  : array[0..maxpec] of Extended ; { power for ecc-ty }

procedure GetRowMem ;

procedure FreeRowMem ;

implementation

procedure GetRowMem ;  { array to heap by pointer }
begin
  memrec:=SizeOf(TRowVar); { size of record TRowVar }
  GetMem(rowone,SizeOf(TRowDim));
  GetMem(rowtwo,SizeOf(TRowDim));
  GetMem(rowtre,SizeOf(TRowDim));
  GetMem(rowfur,SizeOf(TRowDim));
  GetMem(rowfiv,SizeOf(TRowDim));
  GetMem(rowsix,SizeOf(TRowDim));
  GetMem(rowaux,SizeOf(TRowDim));
  GetMem(rowtmp,SizeOf(TRowDim));
  GetMem(rowcoh,SizeOf(TRowDim));
end;

procedure FreeRowMem ; { array to heap by pointer }
begin
  FreeMem(rowcoh,SizeOf(TRowDim));
  FreeMem(rowtmp,SizeOf(TRowDim));
  FreeMem(rowaux,SizeOf(TRowDim));
  FreeMem(rowsix,SizeOf(TRowDim));
  FreeMem(rowfiv,SizeOf(TRowDim));
  FreeMem(rowfur,SizeOf(TRowDim));
  FreeMem(rowtre,SizeOf(TRowDim));
  FreeMem(rowtwo,SizeOf(TRowDim));
  FreeMem(rowone,SizeOf(TRowDim));
end;

end.
