
unit UnVakArg ; { for numerical values of argument }

interface

uses
  UnConTyp , { type TFundArg }
  UnTypVar ; { type TValArg }

procedure ToGetFundamArgu ( td  : Extended ;        // date in julian day 
                            var  ra  : Extended ;   // ERA radian
                            var  ag  : TFundArg ) ; // fundamental argum.

procedure ToGetFunVelArgu ( var  va  : Extended ;   // d(ERA)/dt radian/s
                            var  vf  : TFundArg ) ; // d(f.a)/dt radian/s

procedure ToGetArgumValue ( ul , ug , uh  : Extended ; { l g h angle }
                            ra  : Extended ;           { ERA as S_Z }
                            ag  : TFundArg ;           { type UnConTyp }
                            var  ar  : TValArg );      { type UnTypVar }

procedure ToGetArgumVelos ( vl , vg , vh  : Extended ; { l g h angle }
                            va  : Extended ;           { ERA as S_Z }
                            vf  : TFundArg ;           { type UnConTyp }
                            var  vr  : TValArg );      { type UnTypVar }

implementation

uses
  UnForTyp , { types and global vaiables }
  UnGetERP , { procedure ToObtainERParm for Earth rotation parameters }
  UnRotatM , { procedure ToClcTimeValue to records DatPole DatTime }
  Un2000_r , { for function ToGetEarthRotAngle }
  Un2000_c ; { for procedure FundArgLuSo l l' F D \Omega  \lambda=F+\Omega }

{ td input date in julian day
     output
  numerical value
  of fundamental arguments
  l l' F D \Om ; \la=F+\Om in radian
  and value of Earth Rotating Angle (ERA) in radian }

procedure ToGetFundamArgu ( td  : Extended ;        // date in julian day 
                            var  ra  : Extended ;   // ERA radian
                            var  ag  : TFundArg ) ; // fundamental argum.
begin
  { record DatPole with utc dut xpo ypo dxe dye ToClcTimeValue UnRotatM }
  ToObtainERParm(td,DatPole); { UnGetERP ,  DatPole  from  UnForTyp }
  ToClcTimeValue(td); { result to DatTime , DatPole global UnForTyp }
  { Earth rotation angle as sidereal time }
  ra:=ToGetEarthRotAngle(DatTime.ut1); { function from Un2000_r }
  { fundamental arguments in radian }
  FundArgLuSo(DatTime.tt,ag); { l l' F D \Om ; \la=F+\Om Un2000_c radian }
end;

{ output
  numerical value
  Earth Rotating Angle d(ERA)/dt in radian/s 
  d(l l' F D \Om ; \la=F+\Om)/dt in radian/s }

procedure ToGetFunVelArgu ( var  va  : Extended ;   // d(ERA)/dt radian/s
                            var  vf  : TFundArg ) ; // d(f.a)/dt radian/s
begin
  va:=VelOfRot; { rotation velosity
  { d(fundamental arguments)/dt in radian per second }
  FundVelLuSo(DatTime.tt,vf); { Un2000_c d(argum)/dt }
end;

  { numerical value for arguments 
    1 l=M 2 g=omega+Pi/2  3 h=Omega-Pi/2 in our case
    where
    Omega is the ascending node
    omega is argument of perigei
    M is mean anomaly as usual kepler angle elements
    4 \theta is Earth rotation angle as sidereal time
    5 \lambda is maen longitude of the Moon 
    6 l  mean anomaly of the Moon
    7 l' mean anomaly of the Sun
    8 F  mean argument of latitude of the Moon
    9 D  difference between mean longitude the Moon and the Sun }

procedure ToGetArgumValue ( ul , ug , uh  : Extended ; { l g h angle }
                            ra  : Extended ;           { ERA as S_Z }
                            ag  : TFundArg ;           { type UnConTyp }
                            var  ar  : TValArg );      { type UnTypVar }
begin
  ar[1]:=ul; { mean anomaly radian }
  ar[2]:=ug; { argument of perigei radian }
  ar[3]:=uh; { the ascending node radian }
  ar[4]:=ra; { Earth rotation angle as sidereal time radian }
  { mean longitude of the Moon \lambda fundamental arguments }
  ar[5]:=ag[3]+ag[5]; { \lambda=F+\Omega Un2000_c radian }
  ar[6]:=ag[1]; { l  mean anomaly of the Moon }
  ar[7]:=ag[2]; { l' mean anomaly of the Sun }
  ar[8]:=ag[3]; { F  mean argument of latitude of the Moon }
  { mean longitude of the Moon minus mean longitude of the Sun \lambda-L }
  ar[9]:=ag[4]; { D  difference between mean longitude in radian }
end;

{ velosity for all arguments in radian per second }

procedure ToGetArgumVelos ( vl , vg , vh  : Extended ; { l g h angle }
                            va  : Extended ;           { ERA as S_Z }
                            vf  : TFundArg ;           { type UnConTyp }
                            var  vr  : TValArg );      { type UnTypVar }
begin
  vr[1]:=vl; { dl/dt mean anomaly rad/s }
  vr[2]:=vg; { dg/dt argument of perigei rad/s }
  vr[3]:=vh; { dh/dt the ascending node rad/s }
  vr[4]:=va; { d(ERA)/dt radian per second}
  { mean longitude of the Moon \lambda fundamental arguments }
  vr[5]:=vf[3]+vf[5]; { \lambda=F+\Omega Un2000_c radian }
  vr[6]:=vf[1]; { l  mean anomaly of the Moon }
  vr[7]:=vf[2]; { l' mean anomaly of the Sun }
  vr[8]:=vf[3]; { F  mean argument of latitude of the Moon }
  { mean longitude of the Moon minus mean longitude of the Sun \lambda-L }
  vr[9]:=vf[4]; { D  difference between mean longitude in radian }
end;

end.
