unit UnVakDel ; { to obtain simple derivations numerical values }
interface
var          { global variables simple derivations }
  cang  : Extended ; { canonical element G = L * \sqrt(1-e^2) km^2/s }
  dadl  : Extended ; { da/dL }
  dedl  : Extended ; { e*de/dL }
  dedg  : Extended ; { e*de/dG }
  delg  : Extended ; { de/dL+de/dG }
  dsgs  : Extended ; { ds/dG pure }
  dshs  : Extended ; { ds/dH pure }
  dsdg  : Extended ; { s*ds/dG s=sin(i)}
  dsdh  : Extended ; { s*ds/dH }
  dsgh  : Extended ; { ds/dG+ds/dH }
  dcdg  : Extended ; { s*dc/dG c=cos(i) }
  dcdh  : Extended ; { s*dc/dH }
  dcgh  : Extended ; { dc/dG+dc/dH }
procedure ToGetDerelValue ( aa , ee , si , ci  : Extended ) ;
implementation
uses
  UnTypVar , { global variable signuc signum(cos(i)) }
  UnHarMod ; { for global variable GemParmValue[1..4] fm in [1] }
{ result to global variables
  input numerical values
  aa semimajor axis in km
  ee eccentricity
  si sinus of angle of inclination
  ci cosinus of angle of inclination
     may be negative with signuc = +1 or -1 from UnTypVar }
procedure ToGetDerelValue ( aa , ee , si , ci  : Extended ) ;
var
  fm  : Extended ; { geocentric constant in km^3/s^2 from GemParmValue[1] }
  ql  : Extended ; { canonical element L = \sqrt(fma) }
  qg  : Extended ; { canonical element G = L * \sqrt(1-e^2) }
  qh  : Extended ; { canonical element H = G * cos(i) may be negative }
  re  : Extended ; { numerical value (1-e^2) }
  rx  : Extended ; { numerical value \sqrt(1-e^2) }
  rs  : Extended ; { numerical value (1-s^2) s=sin(i) }
  rz  : Extended ; { numerical value \sqrt(1-s^2) }
begin
  fm:=GemParmValue[1]; { geocentric constant in km^3/s^2 UnHarMod }
  ql:=Sqrt(fm*aa);     { canonical element L = \sqrt(fma) km^2/s }
  re:=Abs(1-Sqr(ee));  { numerical value (1-e^2) but e < 0.5 always }
  rx:=Sqrt(re);        { \sqrt(1-e^2) }
  qg:=ql*rx;           { canonical element G = L * \sqrt(1-e^2) km^2/s }
  qh:=qg*ci;           { canonical element H = G * cos(i) km^2/s }
  cang:=qg;            { canonical element G = L * \sqrt(1-e^2) km^2/s }
  dadl:=(2.0*aa)/ql;    { da/dL=2*a/L }
  dedl:=+re/ql;         { e*de/dL=+(1-e^2)/L }
  dedg:=-re/qg;         { e*de/dG=-(1-e^2)/G }
  { de/dL+de/dG=-(1/L)*(e*\sqrt(1-e^2)/(1+\sqrt(1-e^2)) }
  delg:=-ee*rx/((1+rx)*ql); { de/dL+de/dG }
  rs:=Abs(1-Sqr(si));  { 1-(sin(i))^2 always > 0 }
  rz:=Sqrt(rs);        { \sqrt(1-(sin(i))^2) }
  dsdg:=+rs/qg;        { s*ds/dG=+(1-s^2)/G }
  dsdh:=-signuc*rz/qg; { s*ds/dH=-(1-s^2)/H but H may be small for i=90 }
  if  ( ( ci > 0.0 ) and ( si < 0.5 ) ) { when angle of inclination < 30 }
    then { ds/dG+ds/dH=-(1/G)*(s*\sqrt(1-s^2)/(1+\sqrt(1-s^2)) }
      dsgh:=-si*rz/((1+rz)*qg) { ds/dG+ds/dH for small s }
    else { simple sum (s*ds/dG+s*ds/dH)/s for other s value }
      dsgh:=(dsdg+dsdh)/si;    { ds/dG+ds/dH  other s value }
  if  si > 1.0e-5
    then
      begin            { to select value for sin(i) }
        dsgs:=dsdg/si; { ds/dG=+(1-s^2)/(s*G) }
        dshs:=dsdh/si; { ds/dH=-(1-s^2)/(s*H) }
      end
    else
      begin            { s=sin(i) is too small }
        dsgs:=0.0;
        dshs:=0.0;
      end;
  dcdg:=-qh/Sqr(qg);   { dc/dG=-H/G^2 }
  dcdh:=+1.0/qg;       { dc/dH=+1/G }
  dcgh:=dcdg+dcdh;     { dc/dG+dc/dH }
  dcdg:=si*dcdg;       { s*dc/dG }
  dcdh:=si*dcdh;       { s*dc/dH }
end;
end.
