unit UnVakElm ; { from position and velosity in standard system }
interface       { to nonsingular elements system }
uses
  UnConTyp , { type TVect3 }
  UnForTyp ; { type TVarSix TIntTime }
{ input
  r  position in km
  v  velosity in km/s true equator system
  output nonsingular elements a l+g+h e*cos(g+h) e*sin(g+h)... }
procedure ElemFromPosVel ( r , v   : TVect3 ;    { position velosity UnConTyp }
                           var  e  : TVarSix ) ; { nonsingular elements }
{ to obtain set of simple elements
  by using numerical values of nonsingular elements }
procedure SimpleElem ( ec  : TVarSix ; { set of nonsingular elements }
                       var  aa , ee , si , ci  : Extended ; { a e sin cos i }
                       var  ul , ug , uh  : Extended ) ;   { angle elements }
{ to obtain numerical value of the motion of the longitude
  d(l+g+h)/dt=d(\alpha_1)/dL+d(\alpha_1)/dG+d(\alpha_1)/dH
             =n_0*(1+\nu+\mu)/(1-\lambda) for intermediate orbit }
function ToGetLonVelosity  : Extended ;
{ from elements a e s
  to obtain numerical value of the motion of the longitude
  d(l+g+h)/dt=d(\alpha_1)/dL+d(\alpha_1)/dG+d(\alpha_1)/dH
             =n_0*(1+\nu+\mu)/(1-\lambda) for intermediate orbit }
function ToGetAngMot ( a , e , s  : Extended ) : Extended ;
implementation
uses
  UnTypVar , { global variable signuc Signum(cos(i)) }
  UnForFun , { function DATan2 }
  UnHarMod , { global var GemParmValue[1..4] }
  UnitCeil ; { for class TCeil with methods FromCarToEil }
(*
  procedure ElemFromPosVel
  input
  r  position in km
  v  velosity in km/s true equator system
  output
  nonsingular elements a l+g+h e*cos(g+h) e*sin(g+h) s*cos(h) s*sin(h)
  types from UnConTyp UnForTyp
  TEilParm  = Array [1..90] of Extended ; for UnitCEil class TCeil
  TEilPang  = record                      record with angle
                pelm  : TEilParm ;        parameter
                angl  : Extended ;        angle l radian
                angg  : Extended ;        angle g radian \omega+90 degree
                angh  : Extended ;        angle h radian \Omega-90 degree
              end;
*)
procedure ElemFromPosVel ( r , v   : TVect3 ;    { position velosity UnConTyp }
                           var  e  : TVarSix ) ; { nonsingular elements }
var
  x  : TEilPang ; { parameter and angle UnConTyp }
  l  : Extended ; { angle elements l in radian }
  g  : Extended ; { angle elements g in radian }
  h  : Extended ; { angle elements h in radian }
begin
                              // method class TCeil UnitCeil
  ceiler.FromCarToEil(r,v,x); // result signum(cos(i) and x record
                              // pelm[1..90] angl angg angh in rad
  l:=x.angl; // angle element l in radian
  g:=x.angg; // angle element g in radian
  h:=x.angh; // angle element h in radian
  { nonsingular elements }
  e[1]:=x.pelm[8];          { a  semi-major axis in km }
  e[2]:=l+g+h;              { l+g+h sum of three angles in radian }
  e[3]:=x.pelm[9]*cos(g+h); { e*cos(g+h) }
  e[4]:=x.pelm[9]*sin(g+h); { e*sin(g+h) }
  e[5]:=x.pelm[38]*cos(h);  { \delta*cos(h) }
  e[6]:=x.pelm[38]*sin(h);  { \delta*sin(h) }
end;
{ to obtain set of simple elements
  by using numerical values of nonsingular elements }
procedure SimpleElem ( ec  : TVarSix ; { set of nonsingular elements }
                       var  aa , ee , si , ci  : Extended ; { a e sin cos i }
                       var  ul , ug , uh  : Extended ) ; { angle elements }
var
  c,s,dd  : Extended ;
begin
  c:=ec[5]; { \delta*cos(h) }
  s:=ec[6]; { \delta*sin(h) }
  { \delta = sqrt((\delta*cos(h))^2+(\delta*sin(h))^2) }
  dd:=Sqrt(Sqr(c)+Sqr(s)); { \delta > 0 cos(i) may be negative if i > 90 }
  uh:=DATan2(s,c); { numerical value of the element h function UnForFun }
  c:=ec[3]; { e*cos(g+h) }
  s:=ec[4]; { e*sin(g+h) }
  { e = sqrt((e*cos(g+h))^2+(e*sin(g+h))^2)  e >= 0.0 }
  ee:=Sqrt(Sqr(c)+Sqr(s)); { eccentricity may be small }
  ug:=DATan2(s,c)-uh; { value of element g:=(g+h)-h in radian UnForFun }
  aa:=ec[1];          { semi-major axis a in km }
  ul:=ec[2]-ug-uh;    { value of element l=(l+g+h)-g-h }
  { now we have elements
    a e \delta current values
    l g h      angle elements
    we may obtain some parameters by using method of TCeil class }
  ceiler.ab:=aa; // variables a e \delta
  ceiler.ae:=ee; // from class TCeil UnitCeil
  ceiler.ad:=dd; // to obtain all parameters values
  ceiler.EilConstFromElems ; // result to array ceiler.c[1..90] UnitCeil
  si:=ceiler.c[10]; // s sin(i) > 0 cos(i) may be negative if i > 90
  ci:=ceiler.c[11]; // \alpha cos(i) signuc ShortInt from UnTypVar
end;
{ by using array from class TCeil c[1..90]
  to obtain numerical value of the motion of the longitude
  result to global array tremot from UnTypVar
  tremot[1] dl/dt = n_0/(1-\lambda)      radian/s
  tremot[2] dg/dt = n_0*\nu/(1-\lambda)  radian/s
  tremot[3] dh/dt = n_0*\mu/(1-\lambda)  radian/s
  d(l+g+h)/dt=d(\alpha_1)/dL+d(\alpha_1)/dG+d(\alpha_1)/dH
             =n_0*(1+\nu+\mu)/(1-\lambda) for intermediate orbit }
function ToGetLonVelosity  : Extended ;
begin
  tremot[1]:=ceiler.c[24]; // UnTypVar UnitCeil dl/dt intermediate orbit
  tremot[2]:=ceiler.c[25]; // UnTypVar UnitCeil dg/dt intermediate orbit
  tremot[3]:=ceiler.c[26]; // UnTypVar UnitCeil dh/dt intermediate orbit
  { n_0(1+\nu+\mu)/(1-\lambda) to sum for longitude velosity }
  ToGetLonVelosity:=tremot[1]+tremot[2]+tremot[3]; // sum to UnRightS
end ;
{ from elements a e s
  to obtain numerical value of the motion of the longitude
  result to global array tremot from UnTypVar
  tremot[1] dl/dt = n_0/(1-\lambda)      radian/s
  tremot[2] dg/dt = n_0*\nu/(1-\lambda)  radian/s
  tremot[3] dh/dt = n_0*\mu/(1-\lambda)  radian/s
  d(l+g+h)/dt=d(\alpha_1)/dL+d(\alpha_1)/dG+d(\alpha_1)/dH
             =n_0*(1+\nu+\mu)/(1-\lambda) for intermediate orbit }
function ToGetAngMot ( a , e , s  : Extended ) : Extended ;
var { a semimajor axis e eccentricity s sinus inclination }
  fm,cj2,sig, { geocentric gravitational const f*M parameters c \sigma }
  o,o2,o3,o4,o5,e2,e4,
  re,s2,rs,s4,
  p,p2,ps,p4,
  z2,r1,r2,r3,r4,
  f0,f1,f2,b0,b1,b2        : Extended ;
  aq,a1,ac,ad,d2,dp,ax,
  ak1,ak2,ac0,aq0,ap0,af0  : Extended ;
  an0,alam,ag1,ag2,an,amu  : Extended ;
begin
  fm:=GemParmValue[1];  { global array UnHarMod f*M near 3.986e5 km^2/s^3 }
  cj2:=GemParmValue[3]; { global array UnHarMod parameter c near 200 km }
  sig:=GemParmValue[4]; { global array UnHarMod parameter \sigma near 1/30 }
  ac:=Sqrt(Abs(1.0e0-s*s));      { cosinus inclination for i<=90 }
  if  signuc =-1  then  ac:=-ac; { our inclination i > 90 degree }
  o:=1.0e0;      { simple 1 }
  o2:=2.e0;      { simple 2 }
  o3:=3.e0;      { simple 3 }
  o4:=0.25e0;    { simple 1/4 }
  o5:=0.5e0;     { simple 1/2 }
  e2:=e*e;       { e^2 }
  re:=o-e2;      { 1-e^2 }
  e4:=e2*e2;     { e^4 }
  s2:=s*s;       { (sin(i))^2 }
  rs:=o-s2;      { 1-(sin(i))^2 }
  s4:=s2*s2;     { (sin(i))^4 }
  p:=cj2/(a*re); { \epsilon=c/(a*(1-e^2)) small parameter c close to 1/30 }
  p2:=p*p;       { (\epsilon)^2    near 0.001 }
  ps:=p*sig;     { \epsilon*\sigma near 0.001 }
  z2:=sig*sig;   { (\sigma)^2      near 0.001 }
  ax:=e*(o+p2*re*(o-o2*s2+p2*(o3
          -16.e0*s2+14.e0*s4-o2*e2*rs*rs)));  { parameter \bar e }
  ak1:=0.125e0*p2*s2*(re+z2-4.e0*p2*re*rs);                 { 1/8*k_1^2 }
  ak2:=0.125e0*p2*e2*(s2-p2*(o-10.e0*s2+11.0e0*s4+e2*s4));  { 1/8*k_2^2 }
  p2:=o5*p2;         { (1/2)*(\epsilon)^2 }
  p4:=o5*p2*p2;      { (1/8)*(\epsilon)^4 }
  ag1:=(o+p2*(rs*(o3+e2)+z2*(6.e0-7.e0*s2))        { \sigma_1 without G }
         -p4*rs*(9.e0+11.0e0*s2+e2*(6.e0+34.e0*s2)+e4*(o+o3*s2)));
  ag2:=(o-p2*(o3-4.e0*s2-e2)-p4*(9.e0-72.e0*s2
         +64.e0*s4+e2*(o2-40.e0*s2+48.e0*s4)+e4)); { \sigma_2 without G}
  an:=ag1*(o+o2*ak2+9.e0*Sqr(ak2))/(ag2*(o+o2*ak1+9.e0*Sqr(ak1))); { 1+\nu }
  p2:=p*p;       { (\epsilon)^2 near 0.001 }
  p4:=p2/16.0e0; { (\epsilon)^2/16 }
  r1:=p2*ac;     { (\epsilon)^2*c }
  r2:=o2*p4;     { (\epsilon)^2/8 }
  r3:=4.e0*r2;   { (\epsilon)^2/2 }
  r4:=o4*r1*p4;  { (\epsilon)^4*c/64 }
  ac0:=-r1*(o+o5*e2+p4*(24.e0-56.e0*s2                      { parameter }
             -4.e0*e2*(o+16.e0*s2)-e4*(o2+o3*s2)));         { \alpha'_0 }
  aq0:=-o5*r1*(re+o3*z2                                     { parameter }
                 -o2*p4*re*(30.e0-35.e0*s2+e2*(o2+o3*s2))); { \beta_0 }
  amu:=ac0+an*aq0;                    { \mu=\alpha'_0+(1+\nu)*\beta_0 }
  f2:=o-r3*re*(4.e0-7.e0*s2)-p2*r2*re
       *(20.e0-136.e0*s2+113.e0*s4-e2*(8.e0+24.e0*s2-47.e0*s4));
  f1:=p2*re*(o2-o3*s2+r3*(o2-22.e0*s2+19.e0*s4
               -e2*s2*(10.e0-13.e0*s2)));
  f0:=o/(f1+f2);
  f0:=f0/Sqrt(o-ax*ax);            { simple temporary values }
  f0:=f0*re*re;
  b0:=r3*(-s2+r3*(6.e0-20.e0*s2+15.e0*s4-e2*s4));
  b1:=-p2*r3*s2*(o2-s2);           { simple auxiliary values }
  b2:=o3*p2*r2*s4;
  ap0:=f0*(b0+b1+b2*(o+o5*ax*ax)); { \gamma_0 }
  f0:=p2*f0*ag2/ag1;
  af0:=o5*f0*s2*(o+o3*ak1);        { \gamma'_0 }
  alam:=-ap0-an*af0;  { \lambda=-\gamma_0-(1+\nu)*\gamma'_0 }
  ad:=s-ps*rs*(o-ps*s-p2*(o3-4*s2+e2)); { \delta from s }
  d2:=Sqr(ad);   { (\delta)^2 }
  dp:=p2*d2*re; { (\epsilon)^2*(\delta)^2*(1-e^2) }
  aq:=(o-o2*ps*ad-dp)/(o+o2*p2*d2*(o+e2)+Sqr(dp)); { parameter Q }
  a1:=-(fm/a)*(o-p2*re*(o-d2)*aq); { \2*alpha_1 }
  an0:=-a1*sqrt(-a1)/fm; { n_0=\sqrt((-2*\alpha_1)^3)/(f*M) }
  tremot[1]:=an0/(1.0-alam); { UnTypVar dl/dt intermediate orbit }
  tremot[2]:=(an-1)*tremot[1]; { UnTypVar dg/dt intermediate orbit }
  tremot[3]:=amu*tremot[1]; { UnTypVar dh/dt intermediate orbit }
  ToGetAngMot:=an0*(an+amu)/(1-alam); { n_0(1+\nu+\mu)/(1-\lambda) }
end ;
end.
