unit UnVakFut ; { for inertial forces refer to true equator }
interface       { F = \Omega_x*G*s*cos(h) + \Omega_y*G*s*sin(h) }
uses
  UnTypVar , { for types and pointer array of record rowcoh^[..] }
  UnForTyp ; { for type TVarSix for forces }
{ to sum inertial forces with global variables from UnVakFur }
procedure ToGetForceInert ( ar  : TValArg ;        { array[1..9] UnTypVar }
                            var  fc  : TVarSix ) ; { array[1..6] UnForTyp }
implementation
uses
  UnConTyp , { const DifEpoch = 2400000.5 }
  UnVakNuP , { for value \epsilon d(\Delta\psi)/dt ... }
  UnVakDel ; { for global variables partial derivations }
{ to sum inertial forces with global variables from UnVakFur }
procedure ToGetForceInert ( ar  : TValArg ;        { array[1..9] UnTypVar }
                            var  fc  : TVarSix ) ; { array[1..6] UnForTyp }
var
  tc  : Extended ; { moment in modified julian day }
  ee  : Extended ; { eccentricity for forces }
  si  : Extended ; { sinus inclination for forces }
  cg  : Extended ; { cos(g+h) for forces }
  sg  : Extended ; { sin(g+h) for forces }
  ch  : Extended ; { cos(h) for forces }
  sh  : Extended ; { sin(h) for forces }
  wp  : Extended ; { value d(\Delta\psi)/dt      radian per second }
  we  : Extended ; { value d(\Delta\epsilon)/dt  radian per second }
  ox  : Extended ; { angle velosity x coordinate radian per second }
  oy  : Extended ; { angle velosity y coordinate radian per second }
  op  : Extended ; { +\Omega_x*cos(h)+\Omega_y*sin(h) }
  ou  : Extended ; { -\Omega_x*sin(h)+\Omega_y*cos(h) }
  dg  : Extended ; { dF/dG=(s+G*ds/dG)*(+\Omega_x*cos(h)+\Omega_y*sin(h)) }
  dh  : Extended ; { dF/dH= (G*ds/dH) *(+\Omega_x*cos(h)+\Omega_y*sin(h)) }
  du  : Extended ; { dF/dh=    G*s    *(-\Omega_x*sin(h)+\Omega_y*cos(h)) }
begin
  { current moment in modified julian day }
  tc:=DatTime.teph-DifEpoch; { global UnForTyp value from UnRigthS }
  ClcNutVel(tc,wp,we); { velosity parameter of nutation }
  { \Omega_x = - d(\Delta\epsilon)/dt angle velosity true equator }
  ox:=-we; { angle velosity x radian per second -d(\Delta\epsilon)/dt }
  { \Omega_y = d(\theta)/dt
               + d(\Delta\psi)/dt*cos(\epsilon) angle velosity }
  oy:=ToGetTetaV(tc)+wp*Cos(ToGetEpsiloM(tc)); { UnVakNuP rad./second }
  { some numerical values for our variables }
  ee:=eccmow[1]; { global UnTypVar eccentricity }
  si:=sinmow[1]; { global UnTypVar sinus inclination }
  cg:=Cos(ar[2]+ar[3]); { cos(g+h) }
  sg:=Sin(ar[2]+ar[3]); { sin(g+h) }
  ch:=Cos(ar[3]); { cos(h) }
  sh:=Sin(ar[3]); { sin(h) }
  op:=+ox*ch+oy*sh; { +\Omega_x*cos(h)+\Omega_y*sin(h) }
  ou:=-ox*sh+oy*ch; { -\Omega_x*sin(h)+\Omega_y*cos(h) }
  { dF/dG=(s+G*ds/dG)*(+\Omega_x*cos(h)+\Omega_y*sin(h)) }
  dg:=(si+cang*dsgs)*op; { inertial forces dF/dG derivations UnVakDel }
  { dF/dH= (G*ds/dH) *(+\Omega_x*cos(h)+\Omega_y*sin(h)) }
  dh:=cang*dshs*op;      { inertial forces dF/dH derivations UnVakDel }
  { dF/dh=    G*s    *(-\Omega_x*sin(h)+\Omega_y*cos(h)) }
  du:=cang*si*ou;        { inertial forces dF/dh derivations UnVakDel }
  { to add inertial forces
    to the second equation  d(l+g+h)/dt = ... -dF/dG-dF/dH }
  fc[2]:=fc[2]-dg-dh;            { -dF/dG-dF/dH }
  { to add inertial forces to the third  equation
    d(e*cos(g+h))/dt = ... +e*sin(g+h)*(dF/dG+dF/dH) }
  fc[3]:=fc[3]+ee*sg*(dg+dh);    { +e*sin(g+h)*(dF/dG+dF/dH) }
  { to add inertial forces to the fourth equation
    d(e*sin(g+h))/dt = ... -e*cos(g+h)*(dF/dG+dF/dH) }
  fc[4]:=fc[4]-ee*cg*(dg+dh);    { -e*cos(g+h)*(dF/dG+dF/dH) }
  { to add inertial forces to the fifth  equation
    d(s*cos(h))/dt = ... +(...+ds/dH*dF/dh)*cos(h)+s*(dF/dH)*sin(h) }
  fc[5]:=fc[5]+dshs*du*ch+si*dh*sh; { global derivations from UnVakDel }
  { to add inertial forces to the sixth  equation
    d(s*sin(h))/dt = ... +(...+ds/dH*dF/dh)*sin(h)-s*(dF/dH)*cos(h) }
  fc[6]:=fc[6]+dshs*du*sh-si*dh*ch; { global derivations from UnVakDel }
end;
end.
