unit UnVakNuF ; { for inertial forces refer to true equator }
interface { F = \Omega_x*G*s*cos(h) + \Omega_y*G*s*sin(h) + \Omega_z*H }
uses
  UnTypVar ; { for type TValArg }
var          { global external }
  dfibg  : Extended ; { dF_(inertia)/dG }
  dfibh  : Extended ; { dF_(inertia)/dH }
  dfiah  : Extended ; { dF_(inertia)/dh }
{ inertial forces for derivations }
procedure DerInertia ( ar  : TValArg ) ; { array[1..9] UnTypVar }
implementation
uses
  UnConTyp , { const DifEpoch = 2400000.5 }
  UnForTyp , { global var DatTime of type TDatTime }
  UnVakNuP , { for value \epsilon d(\Delta\psi)/dt ... }
  UnitCeil , { ceiler of class TCeil variables }
  UnitDeil ; { deiler of class TDeil d\delta/dG d\delta/dH }
{ inertial forces for derivations }
procedure DerInertia ( ar  : TValArg ) ; { array[1..9] UnTypVar }
var
  tc  : Extended ; { moment in modified julian day }
  gc  : Extended ; { approximately parameter G }
  dg  : Extended ; { d\delta/dG deiler.ddone[2] }
  dh  : Extended ; { d\delta/dH deiler.ddone[3] }
  si  : Extended ; { approximately sinus inclination }
  ch  : Extended ; { cos(h) for forces }
  sh  : Extended ; { sin(h) for forces }
  wp  : Extended ; { value d(\Delta\psi)/dt      radian per second }
  we  : Extended ; { value d(\Delta\epsilon)/dt  radian per second }
  ox  : Extended ; { angle velosity x coordinate radian per second }
  oy  : Extended ; { angle velosity y coordinate radian per second }
  oz  : Extended ; { angle velosity z coordinate radian per second }
  op  : Extended ; { +\Omega_x*cos(h)+\Omega_y*sin(h) }
  ou  : Extended ; { -\Omega_x*sin(h)+\Omega_y*cos(h) }
begin
  { current moment in modified julian day }
  tc:=DatTime.teph-DifEpoch; { global UnForTyp value from UnRigthS }
  ClcNutVel(tc,wp,we); { velosity parameter of nutation }
  { \Omega_x = - d(\Delta\epsilon)/dt angle velosity true equator }
  ox:=-we; { angle velosity x radian per second -d(\Delta\epsilon)/dt }
  { \Omega_y = d(\theta)/dt
               + d(\Delta\psi)/dt*sin(\epsilon) angle velosity }
  oy:=+ToGetTetaV(tc)+wp*sin(ToGetEpsiloM(tc)); { UnVakNuP rad./second }
  { \Omega_z = -d(z_A)/dt-d(\zeta_A)/dt
               - d(\Delta\psi)/dt*cos(\epsilon) angle velosity }
  oz:=-ToGetZetaV(tc)-wp*Cos(ToGetEpsiloM(tc)); { UnVakNuP rad./second }
  { some numerical values for our variables }
  gc:=ceiler.c[17];     { approximately variable G action UnitCeil }
  dg:=deiler.ddone[2];  { d\delta/dG class TDeil UnitDeil }
  dh:=deiler.ddone[3];  { d\delta/dH class TDeil UnitDeil }
  si:=ceiler.c[10];     { global UnitCeil TCeil sinus inclination }
  ch:=Cos(ar[3]);       { cos(h) }
  sh:=Sin(ar[3]);       { sin(h) }
  op:=+ox*ch+oy*sh;     { +\Omega_x*cos(h)+\Omega_y*sin(h) }
  ou:=-ox*sh+oy*ch;     { -\Omega_x*sin(h)+\Omega_y*cos(h) }

  { dF/dG = (s+G*ds/dG)*(+\Omega_x*cos(h)+\Omega_y*sin(h)) }
  dfibg:=(si+gc*dg)*op; { inertial forces dF/dG derivations UnitDeil }

  { dF/dH = (G*ds/dH)*(+\Omega_x*cos(h)+\Omega_y*sin(h)) +\Omega_z }
  dfibh:=gc*dh*op+oz;   { inertial forces dF/dH derivations UnitDeil }

  { dF/dh = G*s*(-\Omega_x*sin(h)+\Omega_y*cos(h)) }
  dfiah:=gc*si*ou;      { inertial forces dF/dh derivations UnitDeil }

end; // DerInertia result global dfibg dfibh dfiah

end.
