
unit UnVakOsc ; // a try to obtain osculating orbit

interface
uses
  UnForTyp , // for type TVarSix
  UnConTyp ; // for type TVect3 type TFundArg

procedure ToGetOscul ( tc  : Extended ; // current moment in second
                       ec  : TVarSix ;        // middle elements
                       var  de  : TVarSix ;   // to sum short terms
                       var  eo  : TVarSix ) ; // oscul  elements

procedure PosVelOscu ( eo  : TVarSix ;            // UnForTyp
                       var  xx , vv  : TVect3 ) ; // UnConTyp

procedure ShortSolap ( tc  : Extended ; // current moment in second
                       ec  : TVarSix ;        // middle elements
                       var  de  : TVarSix ) ; // to sum short terms

implementation

uses
  UnTypVar , // type TValArg
  UnVakArg , // for procedure ToGetFundamArgu ToGetArgumValue
  UnVakElm , // for procedure SimpleElem function
  UnitCeil , // method FromEilToCar
  UnitDeil , // deiler of TDeil for public ToGetDerSel d(a e \delta)(L G H)
  UnitDelm , // delmer of TDelm for public DerEilParm  d(par)/d(a e \delta)
  UnVakPow , // for procedure ToGetPowerValue to global arrays UnTypVar
  UnVakFup ; // for procedure ToAddShortTerms to oscular by middle elements

{ to obtain
  a set of osculating nonsingular elements
  by simple method to add short periodic terms }

procedure ToGetOscul ( tc  : Extended ; // current moment in second
                       ec  : TVarSix ;        // middle elements
                       var  de  : TVarSix ;   // to sum short terms
                       var  eo  : TVarSix ) ; // oscul  elements
var
  td  : Extended ; // current time in day
  ra  : Extended ; // Earth rotation angle in radian
  va  : Extended ; // d(ERA)/dt in radian per second
  aa  : Extended ; // usual element a km
  ee  : Extended ; // usual element e
  si  : Extended ; // usual element sin(i)
  ci  : Extended ; // usual element cos(i)
  ul  : Extended ; // usual element angle l radian
  ug  : Extended ; // usual element angle g radian
  uh  : Extended ; // usual element angle h radian
  vl  : Extended ; // dl/dt radian per second ceiler.c[24]
  vg  : Extended ; // dg/dt radian per second ceiler.c[25]
  vh  : Extended ; // dh/dt radian per second ceiler.c[26]
  af  : TFundArg ; // UnConTyp fundamental argument [1..14] value
  vf  : TFundArg ; // UnConTyp fundamental argument [1..14] velosity
  ar  : TValArg ;  // UnTypVar value for arguments l,g,h,s_Z,\la,l,l',F,D
  vr  : TValArg ;  // UnTypVar value for d(l,g,h,s_Z,\la,l,l',F,D)/dt
begin

  { current moment in second relative to selected julian day moment }
  td:=IntTime.begday+(IntTime.cursec+tc)/86400.0; // to julian day

  ToGetFundamArgu(td,ra,af); // UnVakArg ERA and  fundamental argum

  ToGetFunVelArgu(va,vf);    // UnVakArg d/dt(ERA fundamental argum

  { from nonsingular to usual elements with signum(cos(i)) }
  SimpleElem(ec,aa,ee,si,ci,ul,ug,uh); // UnVakElm with ceiler of TCeil

  { numerical value for arguments result to ar of TValArg }
  ToGetArgumValue(ul,ug,uh,ra,af,ar);  // UnVakArg l,g,h,s_Z,\l,l,l',F,D

  { numeriacl values to axicur
    to arrays eccmow sinmow cosmow [0..N] }
  ToGetPowerValue(aa,ee,si,ci); // from UnVakPow a e^i s^i c^i for i

  deiler.ToGetDerSel ; // class TDeil UnitDeil for d(a e \delta)/d(L G H)

  delmer.DerEilParm ;  // class TDelm UnitDelm for d(param)/d(a e \delta)

  { numerical value for d(arg)/dt result to vr of TValArg UnVakArg }
  vl:=ceiler.c[24]; // dl/dt=n_0/(1-\lambda)     radian per second
  vg:=ceiler.c[25]; // dg/dt=n_0*\nu/(1-\lambda) radian per second
  vh:=ceiler.c[26]; // dh/dt=n_0*\mu/(1-\lambda) radian per second
  ToGetArgumVelos(vl,vg,vh,va,vf,vr);  // d(l,g,h,s_Z,\l,l,l',F,D)/dt

  { from middle elements to osculating elements with sum short terms }
  ToAddShortTerms(ar,vr,ec,de,eo); // UnVakFup with ceiler deiler delmer

end;

{ from one set of nonsingular elements
    to DesCart position in km velosity in km/s }

procedure PosVelOscu ( eo  : TVarSix ;            // UnForTyp
                       var  xx , vv  : TVect3 ) ; // UnConTyp
var
  aa  : Extended ; // usual element a km
  ee  : Extended ; // usual element e
  si  : Extended ; // usual element sin(i)
  ci  : Extended ; // usual element cos(i)
  ul  : Extended ; // usual element angle l radian
  ug  : Extended ; // usual element angle g radian
  uh  : Extended ; // usual element angle h radian
  el  : TEilPang ; // type UnConTyp param angles
begin

  { from nonsingular to usual elements with signum(cos(i)) }
  SimpleElem(eo,aa,ee,si,ci,ul,ug,uh); // UnVakElm with ceiler

  { in procedure  SimpleElem
    method EilConstFromElems of class TCeil is called
    array of parameters  ceiler.c[1..90] is filled by values }

  el.pelm:=ceiler.c; // array for parameters values
  el.angl:=ul;       // angle element l value radian
  el.angg:=ug;       // angle element g value radian
  el.angh:=uh;       // angle element h value radian

  { input record el
    el.pelm array[1,..90] parameters
    el.angl el.angg el.angh three angles l g h in degree }

  ceiler.FromEilToCar(el,xx,vv); // result km km/s UnitCeil

end;

{ to get short periodic terms
     due to Solar radiation pressure }

procedure ShortSolap ( tc  : Extended ; // current moment in second
                       ec  : TVarSix ;        // middle elements
                       var  de  : TVarSix ) ; // to sum short terms
var
  td  : Extended ; // current time in day
  ra  : Extended ; // Earth rotation angle in radian
  va  : Extended ; // d(ERA)/dt in radian per second
  aa  : Extended ; // usual element a km
  ee  : Extended ; // usual element e
  si  : Extended ; // usual element sin(i)
  ci  : Extended ; // usual element cos(i)
  ul  : Extended ; // usual element angle l radian
  ug  : Extended ; // usual element angle g radian
  uh  : Extended ; // usual element angle h radian
  vl  : Extended ; // dl/dt radian per second ceiler.c[24]
  vg  : Extended ; // dg/dt radian per second ceiler.c[25]
  vh  : Extended ; // dh/dt radian per second ceiler.c[26]
  af  : TFundArg ; // UnConTyp fundamental argument [1..14] value
  vf  : TFundArg ; // UnConTyp fundamental argument [1..14] velosity
  ar  : TValArg ;  // UnTypVar value for arguments l,g,h,s_Z,\la,l,l',F,D
  vr  : TValArg ;  // UnTypVar value for d(l,g,h,s_Z,\la,l,l',F,D)/dt
begin

  { current moment in second relative to selected julian day moment }
  td:=IntTime.begday+(IntTime.cursec+tc)/86400.0; // to julian day

  ToGetFundamArgu(td,ra,af); // UnVakArg ERA and  fundamental argum

  ToGetFunVelArgu(va,vf);    // UnVakArg d/dt(ERA fundamental argum

  { from nonsingular to usual elements with signum(cos(i)) }
  SimpleElem(ec,aa,ee,si,ci,ul,ug,uh); // UnVakElm with ceiler of TCeil

  { numerical value for arguments result to ar of TValArg }
  ToGetArgumValue(ul,ug,uh,ra,af,ar);  // UnVakArg l,g,h,s_Z,\l,l,l',F,D

  { numeriacl values to axicur
    to arrays eccmow sinmow cosmow [0..N] }
  ToGetPowerValue(aa,ee,si,ci); // from UnVakPow a e^i s^i c^i for i

  deiler.ToGetDerSel ; // class TDeil UnitDeil for d(a e \delta)/d(L G H)

  delmer.DerEilParm ;  // class TDelm UnitDelm for d(param)/d(a e \delta)

  { numerical value for d(arg)/dt result to vr of TValArg UnVakArg }
  vl:=ceiler.c[24]; // dl/dt=n_0/(1-\lambda)     radian per second
  vg:=ceiler.c[25]; // dg/dt=n_0*\nu/(1-\lambda) radian per second
  vh:=ceiler.c[26]; // dh/dt=n_0*\mu/(1-\lambda) radian per second
  ToGetArgumVelos(vl,vg,vh,va,vf,vr);  // d(l,g,h,s_Z,\l,l,l',F,D)/dt

  { short periodic terms due to Solar radiation pressure }
  ToGetShortSolap(ar,vr,de); // UnVakFup with ceiler deiler delmer

end;

end.
