
unit UnVakSeq ; { to solve secular equations }

interface

uses
  UnForTyp ; { types TIntTime TVarSix }

procedure ToSolveSecEqu ( ta  : Extended ;  { start  moment in mjd }
                          tb  : Extended ;  { finish moment in mjd }
                          dt  : Extended ;  { step for integration in second }
                          ds  : TIntTime ;  { UnForTyp date and second }
                          el  : TVarSix ) ; { UnForTyp nonsingular elements }

implementation

uses
  UnTypVar , { step for integration in second solves }
  UnConTyp , { some constants }
  UnVakElm , { procedure SimpleElem }
  UnForFun , { function  ATanDegree }
  UnForDat , { function ToGetSimpleDate for julian day }
  UnEverGo , { EverGratorFord procedure }
  UnGetERP , { procedure ToObtainERParm }
  UnRotatM , { procedure ToClcTimeValue }
  Un2000_r , { function ToGetEarthRotAngle }
  UnVakOsc ; { procedure ToGetOscul to add short terms }

{ for the simple elements with IntTime
  input   sa second  ea nonsingular elements }
procedure ToPrintSimple ( sa  : Extended ; ea  : TVarSix ) ;
var
  td  : Extended ; { moment in julian day }
  aa  : Extended ; { semimajor axis }
  ee  : Extended ; { eccentricity }
  ui  : Extended ; { inclination in degree }
  si  : Extended ; { sinus inclination }
  ci  : Extended ; { cosinus inclination }
  ul  : Extended ; { element l }
  ug  : Extended ; { element g }
  uh  : Extended ; { element h }
  ua  : Extended ; { longitude refer to greenwich }
  ra  : Extended ; { Earth rotation angle }
begin
  SimpleElem(ea,aa,ee,si,ci,ul,ug,uh); { UnVakElm }
  ui:=ATanDegree(si,ci); { UnForFun }
  ua:=RadGra*(ul+ug+uh); { UnConTyp longitude in degree }
  { current moment in second relative to selected julian day moment }
  td:=IntTime.begday+(IntTime.cursec+sa)/86400.0; { to julian day }
  { record DatPole with utc dut xpo ypo dxe dye ToClcTimeValue UnRotatM }
  ToObtainERParm(td,DatPole); { UnGetERP ,  DatPole  from  UnForTyp }
  ToClcTimeValue(td); { result to DatTime , DatPole global UnForTyp }
  { Earth rotation angle as sidereal time }
  ra:=ToGetEarthRotAngle(DatTime.ut1); { function from Un2000_r }
  ua:=ua-RadGra*ra; { approximately longitude refer to greenwich }
  while  ua < 0.0  do  ua:=360.0+ua;
  while  ua > 360.0  do  ua:=ua-360.0;
  WriteLn(ToGetSimpleDate(td),
          aa:13:3,ee:13:8,ui:14:6,ua:14:6);
end;
{ to got to start moment of time }
procedure ToStartPoint ( ta  : Extended ;  { start moment in mjd }
                         dt  : Extended ;  { step for integration in second }
                         ds  : TIntTime ;  { UnForTyp date and second }
                         el  : TVarSix ;   { UnForTyp nonsingular elements }
                         var zc  : TIntTime ;  { result date and second }
                         var ze  : TVarSix ) ; { result nonsingular elements }
var
  zd  : Extended ; { current moment in modified julian day }
  qd  : Extended ; { the same as zd }
  zt  : Extended ; { step in second for integration }
  sa  : Extended ; { time in second for the begin moment }
  sb  : Extended ; { time in second for the  end moment }
  ea  : TVarSix ;  { elements for the begin moment in interval }
  eb  : TVarSix ;  { elements for the  end  moment in interval }
  ts  : Extended ; { temporary value in second }
  zz  : Extended ; { temporary value date in julian day }
begin
  zc:=ds; { default the same moment }
  ze:=el; { default the same elements }
  qd:=ds.begday+ds.cursec/86400.0-DifEpoch; { UnForTyp UnConTyp mjd }
  if  ( Abs(ta-qd) < 1.0e-6 ) { ta start moment zd moment for elements }
    then  Exit ;      { no action this is our moment }
  zz:=DifEpoch+ta;    { from modified to julian day for start moment }
  zc.begday:=Int(zz);              { date in julian day result }
  zc.cursec:=86400.0*(zz-Int(zz)); { part of julian day in second result }
  EvorInit ;        { from UnEverGo to start EverGrator }
  EvorCoefNullo ;   { from UnEverGo to start EverGrator }
  if  ta < qd  then { start moment before elements moment }
    begin           { to go backward to moment ta }
      zt:=-dt;      { step in second for integration is negative }
      IntTime:=ds;  { moment for elements for procedure SecuRightHand }
      sa:=0.0;      { second from IntTime moment initial nullo }
      ea:=el;       { elements for this moment }
      sb:=sa;       { default for integration }
      eb:=ea;       { default for integration }
      zd:=qd;       { current moment in modified julian day }
      while  ( ta < zd )  do { step by step in second }
        begin                { backward to start moment ta in mjd }
          EverGratorFord(zt,sa,ea,sb,eb); { UnEverGo one step }
          zd:=zd+zt/86400.0; { the next step backward  zd in mjd }
        end;                 { now moment zd before start moment ta }
      ts:=86400.0*(ta-qd)-sa;   { interval in second from moment sa }
      VarFromPolinom(ts,ea,ze); { result for elements start moment zc }
      Exit ; { result record zc moment record ze elements }
    end;
  if  ta > qd  then { start moment after elements moment }
    begin           { to go forward to moment ta }
      zt:=+dt;      { step in second for integration is positive }
      IntTime:=ds;  { moment for elements for procedure SecuRightHand }
      sa:=0.0;      { second from IntTime moment initial nullo }
      ea:=el;       { elements for this moment }
      sb:=sa;       { default for integration }
      eb:=ea;       { default for integration }
      zd:=qd;       { current moment in modified julian day }
      while  ( zd < ta )  do { step by step in second }
        begin                { forward to start moment ta in mjd }
          EverGratorFord(zt,sa,ea,sb,eb); { UnEverGo one step }
          zd:=zd+zt/86400.0; { the next step forward  zd in mjd }
        end;                 { now moment zd after start moment ta }
      ts:=86400.0*(ta-qd)-sa;   { interval in second from moment sa }
      VarFromPolinom(ts,ea,ze); { result for elements start moment zc }
      Exit ; { result record zc moment record ze elements }
    end;
end; 
{ numerical integration the evolutionary equations }
procedure ToSolveSecEqu ( ta  : Extended ;  { start  moment in mjd }
                          tb  : Extended ;  { finish moment in mjd }
                          dt  : Extended ;  { step for integration in second }
                          ds  : TIntTime ;  { UnForTyp date and second }
                          el  : TVarSix ) ; { UnForTyp nonsingular elements }
var
  zc  : TIntTime ; { current record with date }
  ze  : TVarSix ;  { current record nonsingular elements }
  sa  : Extended ; { time in second for the begin moment }
  sb  : Extended ; { time in second for the  end moment }
  ea  : TVarSix ;  { elements for the begin moment in interval }
  eb  : TVarSix ;  { elements for the  end  moment in interval }
  tc  : Extended ; { current moment in mjd }
  eo  : TVarSix ;  { elements osculating  with short terms add }
begin
  { record IntTime of TIntTime from UnForTyp
    with variables
         begday Extended date with part in julian day
         cursec Extended time in second from begday moment }
  IntTime:=ds; { global variable UnForTyp for proc from UnRightS }
  { start moment ta and elements moment ds may be not the same }
  ToStartPoint(ta,dt,ds,el,zc,ze); { to moment ta now the same zc ta }
  { initial procedures for numerical integration }
  EvorInit ;        { from UnEverGo to start EverGrator }
  EvorCoefNullo ;   { from UnEverGo to start EverGrator }
  tc:=ta;           { to integrate from the start point }
  IntTime:=zc;  { moment for elements for procedure SecuRightHand }
  sa:=0.0;      { second from IntTime moment initial nullo }
  ea:=ze;       { elements for this moment }
  sb:=sa;       { default for integration }
  eb:=ea;       { default for integration }
  ToPrintSimple(sa,ea); { for the simple elements with IntTime }
  while  ( tc < tb )  do  { to go to the finish point }
    begin
      EverGratorFord(dt,sa,ea,sb,eb); { UnEverGo one step }
      ToPrintSimple(sb,eb); { for simple elements with IntTime }
      ToGetOscul(sb,eb,eo); { UnVakOsc to add short terms }
      ToPrintSimple(sb,eo); { for oscula elements with IntTime }
      tc:=tc+dt/86400.0; { the next step forward  tc in mjd }
    end;
end;
end.
