unit unvashad ; { to integrate secular equations with shadow cases }
interface
uses
  UnConTyp , { for types constants }
  UnForTyp ; { for const type TVarSix }
procedure EverGratorShad ( onestep    : extended ; { step in second integr. }
                           var  tbeg  : extended ; { in second from start }
                           var  zerv  : TVarSix ;  { elements before }
                           var  tend  : extended ; { in second from start }
                           var  rezv  : TVarSix ) ; { elements after result }
implementation
uses
  UnitCeil , { for fields of class ceiler c[24] for example }
  UnEverGo , { procedure EverGratorFord to integrate VarFromPolinom }
  UnForPrt , { prtfile global }
  UnVakOsc , { procedure PosVelOscu for position from the elements }
  UnVakSha , { procedure and global variables for shadow }
  UnRights ; { procedure SecuSolarPress for Solar pressure }
{ to try shadow case for current moment }
function BooShamod ( tc  : extended ; { current moment in second }
                     ta  : extended ; { first moment in second }
                     ea  : tvarsix ) : Boolean ;  { set of elements for ta }
var
  ec  : tvarsix ; { current set of elements }
  xc  : tvect3 ;  { current position vector }
  vc  : tvect3 ;  { current velosity vector }
begin
  { in second from the first point of current interval }
  VarFromPolinom(tc-ta,ea,ec); { UnEverGo simple interpolation }
  PosVelOscu(ec,xc,vc); { UnVakOsc position velosity from elements }
  BooShamod:=BooShadow(tc,xc); { UnVakSha to try shadow case }
end;
{ a try to find the first moment
    with more small step in second }
procedure SmallShadOne ( sa  : extended ;      { step in second }
                         fa  : extended ;      { factor for step }
                         ta  : extended ;      { the first moment }
                         tb  : extended ;      { the secod moment }
                         ea  : tvarsix ;       { elements on ta }
                         var  t  : extended ;  { moment in second }
                         var  b  : boolean ) ; { yes or no shadow }
begin     { with more small step in second }
  t:=t-fa*sa; { one previous step backward }
  if  t < ta+1.0e-6  then  t:=ta; { to prevent }
  b:=BooShamod(t,ta,ea); { to try shadow for this moment }
  while  not b  do       { step by step }
    begin                { to try shadow }
      t:=t+sa; { small step to try shadow }
      if  t > tb-1.0e-6  then  Exit ; { out of interval }
      b:=BooShamod(t,ta,ea); { for the next moment }
  end;                    { for while }
end;
{ a try to find the second moment
    with more small step in second }
procedure SmallShadTwo ( sa  : extended ;      { step in second }
                         fa  : extended ;      { factor for step }
                         ta  : extended ;      { the first moment }
                         tb  : extended ;      { the secod moment }
                         ea  : tvarsix ;       { elements on ta }
                         var  t  : extended ;  { moment in second }
                         var  b  : boolean ) ; { yes or no shadow }
begin     { with more small step in second }
  t:=t-fa*sa; { one previous step backward }
  if  t < ta+1.0e-6  then  t:=ta; { to prevent }
  b:=BooShamod(t,ta,ea); { to try shadow for this moment }
  while  b  do       { step by step }
    begin                { to try shadow }
      t:=t+sa; { small step to try shadow }
      if  t > tb-1.0e-6  then  Exit ; { out of interval }
      b:=BooShamod(t,ta,ea); { for the next moment }
  end;                    { for while }
end;
{ to obtain shadow cases
  in our simple interval
  by using EverGrator interpolation }
procedure AllShadowCases ( ta  : extended ;  { start  point in second }
                           tb  : extended ;  { finish point in second }
                           ea  : tvarsix ) ; { start  elements values }
var
  t  : extended ; { current moment in second from start moment }
  s  : extended ; { step in second to find shadow cases }
  f  : extended ; { factor to decrease step value }
  z  : extended ; { small step in second }
  b  : boolean ;  { yes or no shadow cases }
begin
  s:=10.0;      { to find shadow with small step 10 second }
  f:=10.0;      { factor to decrease step value }
  numshadow:=0; { default number of shadow cases global UnVakSha }
  t:=ta;   { to start from the first point in interval }
repeat     { with small step to find shadow cases }
  b:=BooShamod(t,ta,ea); { to try shadow case current moment }
  z:=s/f; { new small step to try }
  if  b  then { a try to find the first moment }
    SmallShadOne(z,f,ta,tb,ea,t,b); { with step 0.1*s in second }
  z:=z/f; { new small step to try }
  if  b  then { a try to find the first moment }
    SmallShadOne(z,f,ta,tb,ea,t,b); { with step 0.01*s }
  if  b  then  { there is a first moment for shadow }
    begin      { case for shadow begins global variables UnVakSha }
      numshadow:=numshadow+1;  { the next case global UnVakSha }
      if  numshadow > maxshadow  then  numshadow:=maxshadow; { to prevent }
      shadowone[numshadow]:=t; { moment when shadow will begin }
     { to do some calculations throw one step of integration }
      while  ( b and ( t <  tb-1.0e-6 ) )  do { simple condition }
        begin     { for calculations }
          t:=t+s; { small step to find the second moment for shadow }
          if  t > tb-1.0e-6  then  t:=tb; { to prevent errors }
          b:=BooShamod(t,ta,ea); { to try shadow case current moment }
        end;      { may be end for shadow or end for interval }
      z:=s/f; { new small step to try }
      if  not b  then { a try to find the second moment }
        SmallShadTwo(z,f,ta,tb,ea,t,b); { with the new small step }
      z:=z/f; { new small step to try }
      if  not b  then { a try to find the second moment }
        SmallShadTwo(z,f,ta,tb,ea,t,b); { with the new small step }
      shadowtwo[numshadow]:=t; { moment when shadow will end }
      if  numshadow > 1 { simple try to do one big step }
        then  { one big step to find the next shadow case }
          t:=t+shadowone[numshadow]-shadowtwo[numshadow-1]-100.0
        else  { one big step to find the next shadow case }
          t:=t+Pi/ceiler.c[24]; { UnitCeil radian per second }
    end;  { case for shadow begins }
  t:=t+s; { the the next moment with small step }
until  ( t > tb-1.0e-6 ) ; { to the second point of interval }
end;
{ more complex secular equation integration
       by taking into account shadow cases }
procedure EverGratorShad ( onestep    : extended ; { step in second integr. }
                           var  tbeg  : extended ; { in second from start }
                           var  zerv  : TVarSix ;  { elements before }
                           var  tend  : extended ; { in second from start }
                           var  rezv  : TVarSix ) ; { elements after result }
var
  ns  : byte ;     { simple count for the shadow cases }
  ks  : byte ;     { simple count for the set of elements }
  ts  : extended ; { moment in second as half of current shadow }
  ds  : extended ; { interval in second for current shadow case }
  es  : tvarsix ;  { set of elements on moment ts by interpolation }
  fs  : tvarsix ;  { current forces due to Solar radiation pressure }
  cs  : tvarsix ;  { corrections to the set of elements due to shadow }
begin
  EverGratorFord(onestep,tbeg,zerv,tend,rezv); { UnEverGo one step }
  AllShadowCases(tbeg,tend,zerv); { to get shadows   one step }
  for ks:=1 to NumberOfVar do  cs[ks]:=0.0; { initial nullo UnForTyp }
  for ns:=1 to numshadow do       { for all shadow cases }
    begin                                    { to sum corrections }
      ts:=0.5*(shadowone[ns]+shadowtwo[ns]); { half of shadow interval }
      VarFromPolinom(ts-tbeg,zerv,es); { UnEverGo simple interpolation }
      SecuSolaPress(ts,es,fs); { UnRights forces on half moment }
      ds:=shadowtwo[ns]-shadowone[ns]; { in second current shadow time }
      for ks:=1 to NumberOfVar do      { for each from six elements }
        begin                           { global UnVakSha UnForTyp }
          shadowcor[ns][ks]:=fs[ks]*ds; { correction due to Solar pressure }
          cs[ks]:=cs[ks]+shadowcor[ns][ks]; { to sum corrections }
        end; { for six elements to sum correction }
    end; { for all shadow to calculate and to sum corrections }
  for ks:=1 to NumberOfVar do  { for six elements }
    rezv[ks]:=rezv[ks]-cs[ks]; { to subtract virtual shadow perturbations }
end;
{ no secular perturbation in shadow case }
procedure CorrectMomElem( tc  : extended ;     { curent moment in second }
                          var  zc  : tvarsix ) ; { elements with correct }
var
  ns  : byte ;     { simple count for the shadow cases }
  ks  : byte ;     { simple count for the set of elements }
  ds  : extended ; { interval in second for current shadow case }
  cs  : tvarsix ;  { corrections to the set of elements due to shadow }
begin
  if  numshadow = 0  then  Exit ; { no shadow no correction }
end;
{ to correct middle elements
     when there are shadow cases }
procedure ElemWithShadow ( tc  : extended ; { curent moment in second }
                           ta  : extended ; { in second from start }
                           za  : TVarSix ;  { elements before }
                           var  zc  : TVarSix ) ; { elements after result }
begin
  VarFromPolinom(tc-ta,za,zc); { UnEverGo simple interpolation }
  CorrectMomElem(tc,zc); { no secular perturbation in shadow case }
end;
end.
