
 { to use file parmaps.pat
   with the approximation for psi epsilon X Y s(t) in ArcSec }
   
 UNIT UnXYSApp ;

 INTERFACE

 Const
   MPolXYS  = 11 ;
   NParXYS  = 5 ;

 Const
   BoolXYS  : Boolean  = FALSE ;

 Type
   TOneXYS  = Array[1..MPolXYS+1] of Double ;
   TAppXYS  = Record
                BegApp  : Double ;
                EndApp  : Double ;
                PosApp  : Array [1..NParXYS] of TOneXYS ;
              End;

 Var
   FileXYS  : File of TAppXYS ; { file parmaps.pat }
   ApprXYS  : TAppXYS ;
   NRecXYS  : LongInt ; { number of records in binary file }
   NCurXYS  : LongInt ; { current number of record that is read }
   InitXYS  : Extended ; { start moment in binary file }
   StepXYS  : Extended ; { step of approximation in binary file }
   ParmXYS  : Array[1..NParXYS] of Extended ; { psi epsilon X Y s(t) }

 Procedure ToOpenFileXYS ( Var  NPos  : LongInt ) ;

 Procedure ChebAppXYS ( t  : Extended ) ;

 Procedure ToReadFileXYS ( TCur       : Extended ;
                           Var  NPos  : LongInt  ) ;

 Procedure XYSFromApprox ( tcur : Extended ;
                           Var  npos : LongInt ;
                           Var  dpsi , deps  : Extended ;
                           Var  xcip , ycip , ceop  : Extended ) ;

 Procedure ToCloseFileXYS ;

 IMPLEMENTATION

 Uses
   UnForApp ; { unit for arrays polc xuzpo polcpo procedure ClcChebPol }

 { to open and to define external variable
      NRecXYS , InitXYS , StepXYS , BoolXYS , ApprXYS }

 Procedure ToOpenFileXYS ( Var  NPos  : LongInt ) ;
 Begin
   NPos:=-1;
   filemode:=0;
   Assign(FileXYS,'parmaps.pat');
 {$I-}
   ReSet(FileXYS);
 {$I+}
   If  IOResult <> 0  Then  Exit ;
   filemode:=2;
   NRecXYS:=FileSize(FileXYS);
   NPos:=0; { start moment of approximation }
   Seek(FileXYS,NPos);
   Read(FileXYS,ApprXYS);
   InitXYS:=ApprXYS.BegApp;
   StepXYS:=ApprXYS.EndApp-ApprXYS.BegApp; {  interval in day }
   NCurXYS:=NPos; { current number of record that is read }
   BoolXYS:=TRUE; { file exists }
 End;

 { to use Chebyshev coefficients of the approximation
   result to array ParmXYS[1..NParXYS] }

 Procedure ChebAppXYS ( t  : Extended ) ;
 Var
   i,j        : Byte;
   sx,tm,x    : Extended;
 Begin
   sx:=0.5*(ApprXYS.BegApp+ApprXYS.EndApp);
   tm:=0.5*(ApprXYS.EndApp-ApprXYS.BegApp);
   x:=(t-sx)/tm;
   ClcChebPol(MPolXYS,x); { from unit UnForApp array polc }
   For j:=1 To NParXYS Do
     Begin
       ParmXYS[j]:=0;
       For i:=1 To MPolXYS+1 Do
         ParmXYS[j]:=ParmXYS[j]+ApprXYS.PosApp[j][i]*polc[i];
     End;
 End;

 { to read file  FileXYS  parmaps.pat
   with the values of the coefficients record ApprXYS
   that appeared from the Chebyshev approximarion process
   and to obtain the values of the parameters on the moment  TCur }

 Procedure ToReadFileXYS ( TCur       : Extended ;
                           Var  NPos  : LongInt  ) ;
 Var
   r  : Extended ;
 Begin
   If  NOT BoolXYS  Then  ToOpenFileXYS(NPos);
   If  NPos < 0  Then  Exit ; { no file with coefficients }
   If  ( ( TCur < ApprXYS.BegApp ) OR ( TCur > ApprXYS.EndApp ) )
     Then
       Begin
         r:=(TCur-InitXYS)/StepXYS;
         NPos:=Trunc(r);
         If  NPos = NCurXYS  Then  Exit; { this record is already in memory }
         If  NPos < NRecXYS
           Then
             Begin
               Seek(FileXYS,NPos);
               Read(FileXYS,ApprXYS);
               StepXYS:=ApprXYS.EndApp-ApprXYS.BegApp;
               NCurXYS:=NPos; { new number of current record }
             End
           Else
             Begin
               NPos:=-1; { error }
               Exit;
             End;
       End;
 End;

 Procedure XYSFromApprox ( tcur : Extended ;
                           Var  npos : LongInt ;
                           Var  dpsi , deps  : Extended ;
                           Var  xcip , ycip , ceop  : Extended ) ;
 Begin
   npos:=-1; { to prevent strange event }
   dpsi:=0.0;
   deps:=0.0;
   xcip:=0.0;
   ycip:=0.0;
   ceop:=0.0;
   ToReadFileXYS(tcur,npos);
   If  npos < 0  Then  Exit ; { no records }
   ChebAppXYS(tcur); { result in array ParmXYS[1..NParXYS] }
   dpsi:=ParmXYS[1]; { nutation longitude ArcSec }
   deps:=ParmXYS[2]; { nutation latitude  ArcSec }
   xcip:=ParmXYS[3]; { celestial intermediate pole x position ArcSec }
   ycip:=ParmXYS[4]; { celestial intermediate pole y position ArcSec }
   ceop:=ParmXYS[5]; { function s(t) ArcSec }
 End;

 Procedure ToCloseFileXYS ;
 Begin
   If  BoolXYS
     Then
       Begin
         Close(FileXYS);
         BoolXYS:= FALSE ;
       End;
 End;

 END.